within ;
package youBot
  package Controller
    package Components
      model P2V "output a signal from a pwm input signal"
        Modelica.Blocks.Interfaces.RealInput inputPWM         annotation (Placement(
              transformation(extent={{-140,-20},{-100,20}}, rotation=0)));
        Modelica.Blocks.Interfaces.RealOutput outputVoltage         annotation (Placement(
              transformation(extent={{100,-10},{120,10}}, rotation=0)));
        parameter Modelica.SIunits.Voltage maximumVoltage = 24
          "voltage source range = (-maximum_voltage) - maximum_voltage";
        parameter Real maximumPWM = 100
          "pwm input range = (-maximum_pwm) - maximum_pwm";
      equation
        outputVoltage = (inputPWM / maximumPWM) * maximumVoltage;
        annotation (Icon(graphics={
              Text(
                extent={{-100,100},{100,60}},
                lineColor={0,0,255},
                textString=
                     "%name"),
              Rectangle(extent={{-100,40},{100,-40}}, lineColor={0,0,255}),
              Text(
                extent={{-60,22},{60,-28}},
                lineColor={0,0,255},
                textString=
                     "P2V")}),    Diagram(graphics={Text(
                extent={{-50,30},{50,-30}},
                lineColor={0,0,255},
                textString=
                     "P2V"), Rectangle(extent={{-100,40},{100,-40}}, lineColor=
                    {0,0,255})}));
      end P2V;

      model Gain2
        import Modelica.Blocks.Interfaces;
        parameter Real k1 "first gain parameter";
        parameter Real k2 "second gain parameter";
        parameter Real threshold "Threshold value";
        Real gain;
        Interfaces.RealInput inputGain2
          annotation (Placement(transformation(extent={{-140,-20},{-100,20}},
                rotation=0)));
        Interfaces.RealOutput outputGain2
          annotation (Placement(transformation(extent={{100,-10},{120,10}},
                rotation=0)));
        Interfaces.RealInput feedbackVelocity
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
      equation
        gain = if (abs(feedbackVelocity)<threshold) and (threshold > 0) then
          k1 + ((abs(feedbackVelocity)/threshold)  * (k2 - k1)) else k2;
        outputGain2 = gain * inputGain2;
        annotation (Diagram(graphics={
              Polygon(points={{-100,100},{-100,-100},{100,0},{-100,100}},
                  lineColor={0,0,225}),
              Ellipse(extent={{-10,10},{10,-10}}, lineColor={0,0,255}),
              Line(points={{0,-100},{0,-10}}, color={0,0,255}),
              Text(
                extent={{-80,60},{0,0}},
                lineColor={0,0,225},
                textString=
                     "k1"),
              Text(
                extent={{-80,0},{0,-60}},
                lineColor={0,0,225},
                textString=
                     "k2")}),Icon(graphics={
              Text(
                extent={{-100,140},{100,100}},
                lineColor={0,0,255},
                textString=
                     "%name"),
              Polygon(points={{-100,100},{-100,-100},{100,0},{-100,100}},
                  lineColor={0,0,225}),
              Ellipse(extent={{-10,10},{10,-10}}, lineColor={0,0,255}),
              Line(points={{0,-100},{0,-10}}, color={0,0,255}),
              Text(
                extent={{-80,60},{0,0}},
                lineColor={0,0,225},
                textString=
                     "k1"),
              Text(
                extent={{-80,0},{0,-60}},
                lineColor={0,0,225},
                textString=
                     "k2")}));
      end Gain2;

      model DiscreteDerivative
        import Modelica.Blocks.Interfaces;
        import Modelica.Blocks.Discrete;
        import Modelica.Blocks.Math;
        parameter Modelica.SIunits.Time SampleTime;
        Interfaces.RealInput inputValue        annotation (Placement(
              transformation(extent={{-140,-20},{-100,20}}, rotation=0)));
        Interfaces.RealOutput outputValue       annotation (Placement(
              transformation(extent={{100,-10},{120,10}}, rotation=0)));
        Discrete.UnitDelay UnitDelay1(samplePeriod=SampleTime)
          annotation (Placement(transformation(extent={{-10,10},{10,30}},
                rotation=0)));
        Math.Add Add1(k1=1, k2=-1)           annotation (Placement(
              transformation(extent={{30,-10},{50,10}}, rotation=0)));
        Discrete.ZeroOrderHold ZeroOrderHold1(samplePeriod=SampleTime)
          annotation (Placement(transformation(extent={{-48,-10},{-28,10}},
                rotation=0)));
      equation
        connect(UnitDelay1.y, Add1.u1) annotation (Line(points={{11,20},{20,20},
                {20,6},{28,6}}, color={0,0,127}));
        connect(Add1.y, outputValue)
                           annotation (Line(points={{51,0},{80.5,0},{80.5,0},{
                110,0}}, color={0,0,127}));
        connect(inputValue, ZeroOrderHold1.u)  annotation (Line(points={{-120,0},
                {-50,0}}, color={0,0,127}));
        connect(ZeroOrderHold1.y, UnitDelay1.u) annotation (Line(points={{-27,0},
                {-20,0},{-20,20},{-12,20}}, color={0,0,127}));
        connect(ZeroOrderHold1.y, Add1.u2) annotation (Line(points={{-27,0},{
                -20,0},{-20,-6},{28,-6}}, color={0,0,127}));
        annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent=
                  {{-100,-100},{100,100}}), graphics),
                             DymolaStoredErrors,
          Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},
                  {100,100}}), graphics={
              Text(
                extent={{-100,40},{100,-40}},
                lineColor={0,0,255},
                textString=
                     "  E(t-1) - E(t)  "),
              Rectangle(extent={{-100,40},{100,-40}}, lineColor={0,0,255}),
              Text(
                extent={{-100,100},{100,60}},
                lineColor={0,0,255},
                textString=
                     "%name")}));
      end DiscreteDerivative;

      model Limiter2 "Clip signal with a threshold and 2 parameter set"
        Real clip;
        import Modelica.Blocks.Interfaces;
        parameter Real k1 "first clip value";
        parameter Real k2 "second clip value";
        parameter Real threshold "Threshold value";
        Interfaces.RealInput inputClip
          annotation (Placement(transformation(extent={{-140,-20},{-100,20}},
                rotation=0)));
        Interfaces.RealOutput outputClip
          annotation (Placement(transformation(extent={{100,-10},{120,10}},
                rotation=0)));
        Interfaces.RealInput feedbackVelocity
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
      equation
        clip = if abs(feedbackVelocity) < threshold and threshold > 0 then
            k1 + ( (abs(feedbackVelocity) / threshold) * (k2 - k1)) else k2;
        outputClip = if abs(inputClip) < clip then inputClip else
                       if (abs(inputClip) > clip and inputClip > 0) then clip else
                       -clip;
        annotation (Diagram(graphics={
              Rectangle(extent={{-100,100},{100,-100}}, lineColor={0,0,255}),
              Text(
                extent={{60,60},{100,20}},
                lineColor={0,0,255},
                textString=
                     "k2"),
              Text(
                extent={{60,100},{100,60}},
                lineColor={0,0,255},
                textString=
                     "k1"),
              Line(points={{0,80},{0,-80}}, color={0,0,255}),
              Line(points={{-80,0},{80,0}}, color={0,0,255}),
              Line(points={{80,60},{20,60},{-20,-60},{-80,-60}}, color={95,95,
                    95}),
              Line(
                points={{80,20},{18,20},{-20,-20},{-80,-20}},
                color={95,95,95},
                pattern=LinePattern.Dot,
                thickness=0.5)}),
                             Icon(graphics={
              Text(
                extent={{-100,140},{100,100}},
                lineColor={0,0,255},
                textString=
                     "%name"),
              Rectangle(extent={{-100,100},{100,-100}}, lineColor={0,0,255}),
              Text(
                extent={{60,60},{100,20}},
                lineColor={0,0,255},
                textString=
                     "k2"),
              Text(
                extent={{60,100},{100,60}},
                lineColor={0,0,255},
                textString=
                     "k1"),
              Line(points={{-80,0},{80,0}}, color={0,0,255}),
              Line(points={{80,60},{20,60},{-20,-60},{-80,-60}}, color={95,95,
                    95}),
              Line(
                points={{80,20},{18,20},{-20,-20},{-80,-20}},
                color={95,95,95},
                pattern=LinePattern.Dot,
                thickness=0.5),
              Line(points={{0,80},{0,-80}}, color={0,0,255})}),
          DymolaStoredErrors);
      end Limiter2;

      model VelocityRampGenerator
        import Modelica.SIunits;
        import Modelica.Blocks.Interfaces;
        parameter SIunits.AngularVelocity maximumVelocity;
        parameter SIunits.AngularAcceleration acceleration;
        parameter SIunits.Angle positionTolerance;
        parameter SIunits.AngularVelocity velocityTolerance;
        parameter SIunits.Angle stoppingDistanceAdjustment = 0.1;
        SIunits.AngularAcceleration setAcceleration;
        SIunits.AngularVelocity velocity;
        SIunits.Angle stoppingDistance;
        parameter SIunits.Time startTime=1
          "Start time when Velocity Ramp Generator is active";
        Boolean positionEndFlag(start=false);
        Boolean positionReachFlag(start=true);
        Interfaces.RealInput feedbackPosition
          annotation (Placement(transformation(extent={{-140,-20},{-100,20}},
                rotation=0)));
        Interfaces.RealOutput setVelocity                     annotation (Placement(
              transformation(extent={{100,-10},{120,10}}, rotation=0)));
      public
        Interfaces.RealInput feedbackVelocity
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
      equation
        positionReachFlag = if abs(feedbackPosition)< positionTolerance and abs(feedbackVelocity)<velocityTolerance then true else false;
        stoppingDistance = (0.5*(feedbackVelocity^2)/acceleration) + stoppingDistanceAdjustment;
        positionEndFlag = if positionReachFlag then false else
            if abs(feedbackPosition) > stoppingDistance then false else true;
        setAcceleration =
                if positionEndFlag then
                   if abs(velocity)<0.001 then 0 else
                      if velocity < 0 then acceleration else -acceleration else
                   if positionReachFlag or time<startTime then 0 else
                    if abs(velocity) < maximumVelocity then
                      if feedbackPosition < 0 then -acceleration else acceleration else 0;
        der(velocity)= setAcceleration;
        velocity = setVelocity;
        when positionReachFlag and abs(velocity)> 0 then
         reinit(velocity,0);
        end when;
        annotation (extent=[12,-38; 32,-18],
          Diagram(graphics),
          Icon(graphics={
              Line(points={{-60,60},{-60,-60},{60,-60}}, color={0,0,255}),
              Line(points={{-40,-60},{-20,0},{20,0},{40,-60}}, color={0,0,0}),
              Text(
                extent={{-100,140},{100,100}},
                lineColor={0,0,255},
                textString=
                     "%name"),
              Rectangle(extent={{-100,100},{100,-100}}, lineColor={0,0,255})}),
          Placement(transformation(extent={{12,-38},{32,-18}}, rotation=0)),
                    Diagram);
      end VelocityRampGenerator;

    end Components;

    package PIDs
      model PositionPID
        import Modelica.SIunits;
        import youBot.Controller.Components;
        Modelica.Blocks.Interfaces.RealOutput setVelocity
                                                annotation (Placement(
              transformation(extent={{200,-20},{240,20}}, rotation=0)));
      public
        parameter Real gearRatio = 1/156;
        parameter Integer encoderTickPerRound = 4000;
        parameter SIunits.AngularVelocity maximumVelocity = 10
          "rad/s, maximum allowable velocity";
        parameter SIunits.Time PIDPeriod = 0.001 "s, pid time period";
        parameter Modelica.SIunits.AngularVelocity threshold = 0 "(rad/s)";
        parameter Integer IClip1 = 0 "Integral Clip Limit 1";
        parameter Integer IClip2 = 0 "Integral Clip Limit 2";
        parameter Integer PParam1 = 0 "Proportional gain 1";
        parameter Integer PParam2 = 0 "Proportional gain 2";
        parameter Integer IParam1 = 0 "Integral gain 1";
        parameter Integer IParam2 = 0 "Integral gain 2";
        parameter Integer DParam1 = 0 "Derivative gain 1";
        parameter Integer DParam2 = 0 "Derivative gain 2";

        Components.Gain2 integralGain(
          threshold=threshold,
          k1=IParam1,
          k2=IParam2)          annotation (Placement(transformation(extent={{20,
                  -10},{40,10}}, rotation=0)));
        Components.Gain2 derivativeGain(
          threshold=threshold,
          k1=DParam1,
          k2=DParam2)          annotation (Placement(transformation(extent={{20,
                  -50},{40,-30}}, rotation=0)));
        Modelica.Blocks.Math.Add3 sum annotation (Placement(transformation(
                extent={{80,-10},{100,10}}, rotation=0)));
        Modelica.Blocks.Interfaces.RealInput positionError
          annotation (Placement(transformation(extent={{-240,-20},{-200,20}},
                rotation=0)));
        Components.Gain2 proportionalGain(
          threshold=threshold,
          k1=PParam1,
          k2=PParam2)  annotation (Placement(transformation(extent={{-10,30},{
                  10,50}}, rotation=0)));
        Modelica.Blocks.Math.Gain gearBoxNormalization(k=2/gearRatio*Modelica.
              Constants.pi) annotation (Placement(transformation(extent={{120,
                  -10},{140,10}}, rotation=0)));
        Modelica.Blocks.Nonlinear.Limiter positionErrorLimiter(uMax=(65535/
              encoderTickPerRound)*2/gearRatio*Modelica.Constants.pi, uMin=-
              positionErrorLimiter.uMax)
          annotation (Placement(transformation(extent={{-180,-10},{-160,10}},
                rotation=0)));
        Modelica.Blocks.Nonlinear.Limiter setVelocityLimiter(uMax=maximumVelocity,
            uMin=-setVelocityLimiter.uMax)
          annotation (Placement(transformation(extent={{160,-10},{180,10}},
                rotation=0)));
        Components.DiscreteDerivative discreteDerivative(SampleTime=PIDPeriod)
          annotation (Placement(transformation(extent={{-60,-50},{-40,-30}},
                rotation=0)));
        Modelica.Blocks.Interfaces.RealInput velocityFeedback
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
        Components.Limiter2 integralLimiter2(
          threshold=threshold,
          k1=IClip1*0.001*maximumVelocity,
          k2=IClip2*0.001*maximumVelocity)
                               annotation (Placement(transformation(extent={{
                  -40,-10},{-20,10}}, rotation=0)));
        Modelica.Blocks.Continuous.Integrator integrator
          annotation (Placement(transformation(extent={{-100,-10},{-80,10}},
                rotation=0)));
      equation
        connect(sum.y, gearBoxNormalization.u)
          annotation (Line(points={{101,0},{118,0}}, color={0,0,127}));
        connect(gearBoxNormalization.y, setVelocityLimiter.u)
          annotation (Line(points={{141,0},{158,0}}, color={0,0,127}));
        connect(setVelocityLimiter.y, setVelocity)   annotation (Line(points={{
                181,0},{220,0}}, color={0,0,127}));

        connect(positionError, positionErrorLimiter.u)    annotation (Line(
              points={{-220,0},{-182,0}}, color={0,0,127}));
        connect(positionErrorLimiter.y, integrator.u)    annotation (Line(
              points={{-159,0},{-102,0}}, color={0,0,127}));
        connect(integrator.y, integralLimiter2.inputClip)
                                                     annotation (Line(points={{
                -79,0},{-42,0}}, color={0,0,127}));
        connect(velocityFeedback, integralLimiter2.feedbackVelocity)
                                                                annotation (Line(
              points={{0,-120},{0,-20},{-30,-20},{-30,-12}}, color={0,0,127}));
        connect(positionErrorLimiter.y, proportionalGain.inputGain2)
          annotation (Line(points={{-159,0},{-120,0},{-120,40},{-12,40}}, color=
               {0,0,127}));
        connect(proportionalGain.outputGain2, sum.u1)  annotation (Line(points=
                {{11,40},{60,40},{60,8},{78,8}}, color={0,0,127}));
        connect(velocityFeedback, proportionalGain.feedbackVelocity)
          annotation (Line(points={{0,-120},{0,28}}, color={0,0,127}));
        connect(integralLimiter2.outputClip, integralGain.inputGain2)
                                                                 annotation (Line(
              points={{-19,0},{18,0}}, color={0,0,127}));
        connect(integralGain.outputGain2, sum.u2)  annotation (Line(points={{41,
                0},{78,0}}, color={0,0,127}));
        connect(velocityFeedback, integralGain.feedbackVelocity)   annotation (Line(
              points={{0,-120},{0,-20},{30,-20},{30,-12}}, color={0,0,127}));
        connect(velocityFeedback, derivativeGain.feedbackVelocity)
          annotation (Line(points={{0,-120},{0,-60},{30,-60},{30,-52}}, color={
                0,0,127}));
        connect(derivativeGain.outputGain2, sum.u3)  annotation (Line(points={{
                41,-40},{60,-40},{60,-8},{78,-8}}, color={0,0,127}));
        connect(positionErrorLimiter.y, discreteDerivative.inputValue)
          annotation (Line(points={{-159,0},{-120,0},{-120,-40},{-62,-40}},
              color={0,0,127}));
        connect(discreteDerivative.outputValue, derivativeGain.inputGain2)
          annotation (Line(points={{-39,-40},{18,-40}}, color={0,0,127}));
        annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{
                  -200,-100},{200,100}}), graphics={
              Rectangle(extent={{-200,100},{200,-100}}, lineColor={0,0,255}),
              Text(
                extent={{-198,138},{200,100}},
                lineColor={0,0,255},
                textString=
                     "%name"),
              Text(
                extent={{-80,80},{80,0}},
                lineColor={0,0,255},
                textString=
                     "PID"),
              Text(
                extent={{-80,0},{80,-80}},
                lineColor={0,0,255},
                textString=
                     "Position")}),
                          Diagram(coordinateSystem(preserveAspectRatio=false,
                extent={{-200,-100},{200,100}}), graphics),
          DymolaStoredErrors);
      end PositionPID;

      model VelocityPID
        import Modelica.SIunits;
        import youBot.Controller.Components;
        Modelica.Blocks.Interfaces.RealInput setVelocity
          annotation (Placement(transformation(extent={{-240,-20},{-200,20}},
                rotation=0)));
        Modelica.Blocks.Interfaces.RealInput velocityFeedback
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
        Modelica.Blocks.Math.Feedback feedback1 annotation (Placement(
              transformation(extent={{-170,-10},{-150,10}}, rotation=0)));
        Modelica.Blocks.Interfaces.RealOutput setCurrent
                                                annotation (Placement(
              transformation(extent={{200,-20},{240,20}}, rotation=0)));
      public
        parameter SIunits.Time PIDPeriod = 0.05
          "Time period for PID controller";
        parameter Modelica.SIunits.Current maximumCurrent = 2.68
          "maximum allowable current set";
        parameter Modelica.SIunits.AngularVelocity threshold = 0
          "Velocity limit for PID controller parameter";
        parameter Integer IClip1 "Integral Clip Limit 1";
        parameter Integer IClip2 "Integral Clip Limit 2";
        parameter Integer PParam1 "Proportional gain 1";
        parameter Integer PParam2 "Proportional gain 2";
        parameter Integer IParam1 "Integral gain 1";
        parameter Integer IParam2 "Integral gain 2";
        parameter Integer DParam1 "Derivative gain 1";
        parameter Integer DParam2 "Derivative gain 2";
        Components.Gain2 derivativeGain(
          threshold=threshold,
          k1=DParam1,
          k2=DParam2)           annotation (Placement(transformation(extent={{
                  20,-50},{40,-30}}, rotation=0)));
        Components.Gain2 integralGain(
          threshold=threshold,
          k1=IParam1,
          k2=IParam2)           annotation (Placement(transformation(extent={{
                  20,-10},{40,10}}, rotation=0)));
        Components.Gain2 proportionalGain(
          threshold=threshold,
          k1=PParam1,
          k2=PParam2)
          annotation (Placement(transformation(extent={{-10,30},{10,50}},
                rotation=0)));
        Modelica.Blocks.Math.Add3 sum annotation (Placement(transformation(
                extent={{80,-10},{100,10}}, rotation=0)));
        Modelica.Blocks.Math.Gain currentNormalization(k=0.001)
          annotation (Placement(transformation(extent={{120,-10},{140,10}},
                rotation=0)));
        Modelica.Blocks.Nonlinear.Limiter currentSetLimiter(uMax=maximumCurrent, uMin=-
              currentSetLimiter.uMax)
          annotation (Placement(transformation(extent={{160,-10},{180,10}},
                rotation=0)));
        Components.DiscreteDerivative discreteDerivative(SampleTime=PIDPeriod)
                             annotation (Placement(transformation(extent={{-60,
                  -50},{-40,-30}}, rotation=0)));
        Components.Limiter2 integralLimiter2(
          k1=IClip1*0.001*maximumCurrent,
          k2=IClip2*0.001*maximumCurrent,
          threshold=threshold) annotation (Placement(transformation(extent={{
                  -40,-10},{-20,10}}, rotation=0)));
        Modelica.Blocks.Continuous.Integrator integrator
          annotation (Placement(transformation(extent={{-80,-10},{-60,10}},
                rotation=0)));
      equation
        connect(velocityFeedback,feedback1. u2)  annotation (Line(points={{0,
                -120},{0,-60},{-160,-60},{-160,-8}}, color={0,0,127}));
        connect(sum.y, currentNormalization.u)
          annotation (Line(points={{101,0},{118,0}}, color={0,0,127}));
        connect(currentNormalization.y, currentSetLimiter.u)
          annotation (Line(points={{141,0},{158,0}}, color={0,0,127}));
        connect(currentSetLimiter.y, setCurrent)
                                            annotation (Line(points={{181,0},{
                220,0}}, color={0,0,127}));
        connect(setVelocity,feedback1. u1)     annotation (Line(points={{-220,0},
                {-168,0}}, color={0,0,127}));
        connect(feedback1.y, integrator.u)  annotation (Line(points={{-151,0},{
                -82,0}}, color={0,0,127}));
        connect(integrator.y, integralLimiter2.inputClip)
                                                     annotation (Line(points={{
                -59,0},{-42,0}}, color={0,0,127}));
        connect(velocityFeedback, integralLimiter2.feedbackVelocity)
                                                                annotation (Line(
              points={{0,-120},{0,-20},{-30,-20},{-30,-12}}, color={0,0,127}));
        connect(feedback1.y, proportionalGain.inputGain2)  annotation (Line(
              points={{-151,0},{-120,0},{-120,40},{-12,40}}, color={0,0,127}));
        connect(integralLimiter2.outputClip, integralGain.inputGain2)
                                                                 annotation (Line(
              points={{-19,0},{18,0}}, color={0,0,127}));
        connect(integralGain.outputGain2, sum.u2)  annotation (Line(points={{41,
                0},{78,0}}, color={0,0,127}));
        connect(proportionalGain.outputGain2, sum.u1)  annotation (Line(points=
                {{11,40},{60,40},{60,8},{78,8}}, color={0,0,127}));
        connect(velocityFeedback, proportionalGain.feedbackVelocity)
          annotation (Line(points={{0,-120},{0,28}}, color={0,0,127}));
        connect(velocityFeedback, derivativeGain.feedbackVelocity)
          annotation (Line(points={{0,-120},{0,-60},{30,-60},{30,-52}}, color={
                0,0,127}));
        connect(velocityFeedback, integralGain.feedbackVelocity)   annotation (Line(
              points={{0,-120},{0,-20},{30,-20},{30,-12}}, color={0,0,127}));
        connect(derivativeGain.outputGain2, sum.u3)  annotation (Line(points={{
                41,-40},{60,-40},{60,-8},{78,-8}}, color={0,0,127}));
        connect(feedback1.y, discreteDerivative.inputValue)  annotation (Line(
              points={{-151,0},{-120,0},{-120,-40},{-62,-40}}, color={0,0,127}));
        connect(discreteDerivative.outputValue, derivativeGain.inputGain2)
          annotation (Line(points={{-39,-40},{18,-40}}, color={0,0,127}));
        annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent=
                  {{-200,-100},{200,100}}), graphics),
                                         Icon(coordinateSystem(
                preserveAspectRatio=false, extent={{-200,-100},{200,100}}),
              graphics={
              Rectangle(extent={{-200,100},{200,-100}}, lineColor={0,0,255}),
              Text(
                extent={{-80,80},{80,0}},
                lineColor={0,0,255},
                textString=
                     "PID"),
              Text(
                extent={{-80,0},{80,-80}},
                lineColor={0,0,255},
                textString=
                     "Velocity"),
              Text(
                extent={{-198,138},{200,100}},
                lineColor={0,0,255},
                textString=
                     "%name")}),
          DymolaStoredErrors);
      end VelocityPID;

      model CurrentPID
        import Modelica.SIunits;
        import youBot.Controller.Components;
        Modelica.Blocks.Math.Feedback feedback  annotation (Placement(
              transformation(extent={{-180,-10},{-160,10}}, rotation=0)));
        Modelica.Blocks.Interfaces.RealInput setCurrent
          annotation (Placement(transformation(extent={{-240,-20},{-200,20}},
                rotation=0)));
        Modelica.Blocks.Interfaces.RealInput currentFeedback
          annotation (Placement(transformation(
              origin={0,-120},
              extent={{-20,-20},{20,20}},
              rotation=90)));
        Modelica.Blocks.Interfaces.RealOutput setPWM
                                                annotation (Placement(
              transformation(extent={{200,-20},{240,20}}, rotation=0)));
      public
        parameter SIunits.Time PIDPeriod = 0.05
          "Time period for PID controller";
        parameter Integer maximumPWM = 32767 "PWM Input Limiter";
        parameter Integer IClip = 1000 "Integral Clip Limit";
        parameter Integer PParam = 4000 "Proportional gain";
        parameter Integer IParam = 4000 "Integral gain";
        parameter Integer DParam = 0 "Derivative gain";
        Modelica.Blocks.Math.Add3 sum annotation (Placement(transformation(
                extent={{80,-10},{100,10}}, rotation=0)));
        Components.DiscreteDerivative DiscreteDerivative1(SampleTime=PIDPeriod)
          annotation (Placement(transformation(extent={{-60,-50},{-40,-30}},
                rotation=0)));
        Modelica.Blocks.Nonlinear.Limiter PWMLimiter(uMax=maximumPWM, uMin=-PWMLimiter.
              uMax)
          annotation (Placement(transformation(extent={{130,-10},{150,10}},
                rotation=0)));
        Modelica.Blocks.Math.Gain currentNormalization(k=1000)
          annotation (Placement(transformation(extent={{-130,-10},{-110,10}},
                rotation=0)));
        Modelica.Blocks.Math.Gain integralGain(k=IParam)
          annotation (Placement(transformation(extent={{20,-10},{40,10}},
                rotation=0)));
        Modelica.Blocks.Math.Gain derivativeGain(k=DParam)
          annotation (Placement(transformation(extent={{20,-50},{40,-30}},
                rotation=0)));
        Modelica.Blocks.Math.Gain proportionalGain(k=PParam)
          annotation (Placement(transformation(extent={{-6,30},{14,50}},
                rotation=0)));
        Modelica.Blocks.Nonlinear.Limiter integralLimiter(uMax=IClip*0.001*maximumPWM,
            uMin=-integralLimiter.uMax)
          annotation (Placement(transformation(extent={{-34,-10},{-14,10}},
                rotation=0)));
        Modelica.Blocks.Continuous.Integrator Integrator1
          annotation (Placement(transformation(extent={{-80,-10},{-60,10}},
                rotation=0)));
      equation
        connect(currentFeedback, feedback.u2)    annotation (Line(points={{0,
                -120},{0,-80},{-170,-80},{-170,-8}}, color={0,0,127}));
        connect(sum.y, PWMLimiter.u)
                                   annotation (Line(points={{101,0},{128,0}},
              color={0,0,127}));
        connect(PWMLimiter.y, setPWM)   annotation (Line(points={{151,0},{220,0}},
              color={0,0,127}));
        connect(feedback.y, currentNormalization.u)   annotation (Line(points={
                {-161,0},{-132,0}}, color={0,0,127}));
        connect(derivativeGain.y, sum.u3)  annotation (Line(points={{41,-40},{
                60,-40},{60,-8},{78,-8}}, color={0,0,127}));
        connect(integralGain.y, sum.u2)
          annotation (Line(points={{41,0},{78,0}}, color={0,0,127}));
        connect(currentNormalization.y, proportionalGain.u)   annotation (Line(
              points={{-109,0},{-100,0},{-100,40},{-8,40}}, color={0,0,127}));
        connect(proportionalGain.y, sum.u1)  annotation (Line(points={{15,40},{
                60,40},{60,8},{78,8}}, color={0,0,127}));
        connect(integralLimiter.y, integralGain.u)   annotation (Line(points={{
                -13,0},{18,0}}, color={0,0,127}));
        connect(Integrator1.y, integralLimiter.u)  annotation (Line(points={{
                -59,0},{-36,0}}, color={0,0,127}));
        connect(currentNormalization.y, Integrator1.u)  annotation (Line(points=
               {{-109,0},{-82,0}}, color={0,0,127}));
        connect(setCurrent, feedback.u1)      annotation (Line(points={{-220,0},
                {-178,0}}, color={0,0,127}));
        connect(currentNormalization.y, DiscreteDerivative1.inputValue)
          annotation (Line(points={{-109,0},{-100,0},{-100,-40},{-62,-40}},
              color={0,0,127}));
        connect(DiscreteDerivative1.outputValue, derivativeGain.u)  annotation (Line(
              points={{-39,-40},{18,-40}}, color={0,0,127}));
        annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent=
                  {{-200,-100},{200,100}}), graphics),
                                         Icon(coordinateSystem(
                preserveAspectRatio=false, extent={{-200,-100},{200,100}}),
              graphics={
              Rectangle(extent={{-200,100},{200,-100}}, lineColor={0,0,255}),
              Text(
                extent={{-80,80},{80,0}},
                lineColor={0,0,255},
                textString=
                     "PID"),
              Text(
                extent={{-80,0},{80,-80}},
                lineColor={0,0,255},
                textString=
                     "Current"),
              Text(
                extent={{-198,138},{200,100}},
                lineColor={0,0,255},
                textString=
                     "%name")}));
      end CurrentPID;

    end PIDs;

    package Modes
      model PositionMode
        import Modelica.SIunits;
        import Modelica.Blocks.Interfaces;
        import youBot;
        Interfaces.RealInput positionFeedback
          annotation (Placement(transformation(
              origin={-120,-80},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        Interfaces.RealInput velocityFeedback
          annotation (Placement(transformation(
              origin={0,-80},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        Interfaces.RealInput currentFeedback
          annotation (Placement(transformation(
              origin={90,-80},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        youBot.Controller.PIDs.PositionPID positionPID(
          threshold=thresholdPosition,
          IClip1=IClip1Position,
          IClip2=IClip2Position,
          PParam1=PParam1Position,
          PParam2=PParam2Position,
          IParam1=IParam1Position,
          IParam2=IParam2Position,
          DParam1=DParam1Position,
          DParam2=DParam2Position,
          encoderTickPerRound=encoderTickPerRound,
          maximumVelocity=maximumVelocity,
          gearRatio=gearRatio,
          PIDPeriod=PIDPeriodPosition)
                                 annotation (Placement(transformation(extent={{
                  -90,-10},{-50,10}}, rotation=0)));
        youBot.Controller.PIDs.VelocityPID velocityPID(
          PParam1=PParam1Velocity,
          PParam2=PParam2Velocity,
          IClip1=IClip1Velocity,
          IClip2=IClip2Velocity,
          IParam1=IParam1Velocity,
          IParam2=IParam2Velocity,
          DParam1=DParam1Velocity,
          DParam2=DParam2Velocity,
          threshold=thresholdVelocity,
          maximumCurrent=maximumCurrent,
          PIDPeriod=PIDPeriodVelocity)
                                 annotation (Placement(transformation(extent={{
                  -20,-10},{20,10}}, rotation=0)));
        youBot.Controller.PIDs.CurrentPID currentPID(
          maximumPWM=maximumPWM,
          PIDPeriod=PIDPeriodCurrent,
          IClip=IClipCurrent,
          PParam=PParamCurrent,
          IParam=IParamCurrent,
          DParam=DParamCurrent)
                               annotation (Placement(transformation(extent={{70,
                  -10},{110,10}}, rotation=0)));
        Interfaces.RealOutput setVoltage
          annotation (Placement(transformation(extent={{170,-10},{190,10}},
                rotation=0)));
        youBot.Controller.Components.P2V P2V1(maximumVoltage=motorVoltage,
            maximumPWM=maximumPWM)
                 annotation (Placement(transformation(extent={{130,-10},{150,10}},
                rotation=0)));

      public
        parameter Real gearRatio = 156 "Gearbox Ratio";
        parameter Integer encoderTickPerRound = 4000 "Motor Encoder Tick";

        parameter SIunits.AngularVelocity thresholdPosition
          "|Position Controller|| threshold velocity for position controller";
        parameter SIunits.AngularVelocity maximumVelocity = 2
          "|Position Controller|| maximum allowable velocity set";
        parameter SIunits.Time PIDPeriodPosition = 0.001
          "|Position Controller|| Time period for position pid controller";
        parameter Integer PParam1Position
          "|Position Controller|| first parameter for proportional gain in position controller";
        parameter Integer PParam2Position
          "|Position Controller|| second parameter for proportional gain in position controller";
        parameter Integer IParam1Position
          "|Position Controller|| first parameter for integral gain in position controller";
        parameter Integer IParam2Position
          "|Position Controller|| second parameter for integral gain in position controller";
        parameter Integer DParam1Position
          "|Position Controller|| first parameter for derivative gain in position controller";
        parameter Integer DParam2Position
          "|Position Controller|| second parameter for derivative gain in position controller";
        parameter Integer IClip1Position
          "|Position Controller|| first parameter for integral clip in position controller";
        parameter Integer IClip2Position
          "|Position Controller|| second parameter for integral clip in position controller";

        parameter SIunits.AngularVelocity thresholdVelocity
          "|Velocity Controller|| threshold velocity for velocity controller";
        parameter Modelica.SIunits.Current maximumCurrent = 2.68
          "|Velocity Controller|| maximum allowable current set";
        parameter SIunits.Time PIDPeriodVelocity = 0.001
          "|Velocity Controller|| Time period for velocity pid controller";
        parameter Integer PParam1Velocity
          "|Velocity Controller|| first parameter for proportional gain in velocity controller";
        parameter Integer PParam2Velocity
          "|Velocity Controller|| second parameter for proportional gain in velocity controller";
        parameter Integer IParam1Velocity
          "|Velocity Controller|| first parameter for integral gain in velocity controller";
        parameter Integer IParam2Velocity
          "|Velocity Controller|| second parameter for integral gain in velocity controller";
        parameter Integer DParam1Velocity
          "|Velocity Controller|| first parameter for derivative gain in velocity controller";
        parameter Integer DParam2Velocity
          "|Velocity Controller|| second parameter for derivative gain in velocity controller";
        parameter Integer IClip1Velocity
          "|Velocity Controller|| first parameter for integral clip in velocity controller";
        parameter Integer IClip2Velocity
          "|Velocity Controller|| second parameter for integral clip in velocity controller";

        parameter Integer maximumPWM = 32767
          "|Current Controller|| maximum allowable PWM set";
        parameter SIunits.Time PIDPeriodCurrent = 0.001
          "|Current Controller|| Time period for current pid controller";
        parameter Integer PParamCurrent
          "|Current Controller|| proportional gain in current controller";
        parameter Integer IParamCurrent
          "|Current Controller|| integral gain in current controller";
        parameter Integer DParamCurrent
          "|Current Controller|| derivative gain in current controller";
        parameter Integer IClipCurrent = 0
          "|Current Controller|| integral clip in current controller";

        parameter Real motorVoltage "(V) Source voltage";

        Modelica.Blocks.Interfaces.RealInput setPosition
          annotation (Placement(transformation(extent={{-190,-10},{-170,10}},
                rotation=0)));
        Modelica.Blocks.Math.Feedback feedback  annotation (Placement(
              transformation(extent={{-130,-10},{-110,10}}, rotation=0)));
      equation
        connect(setPosition, feedback.u1)     annotation (Line(points={{-180,0},
                {-128,0}}, color={0,0,127}));
        connect(positionFeedback, feedback.u2)   annotation (Line(points={{-120,
                -80},{-120,-8}}, color={0,0,127}));
        connect(P2V1.outputVoltage, setVoltage)     annotation (Line(points={{
                151,0},{180,0}}, color={0,0,127}));
        connect(feedback.y, positionPID.positionError) annotation (Line(points=
                {{-111,0},{-92,0}}, color={0,0,127}));
        connect(velocityFeedback, positionPID.velocityFeedback) annotation (Line(
              points={{0,-80},{0,-40},{-70,-40},{-70,-12}}, color={0,0,127}));
        connect(positionPID.setVelocity, velocityPID.setVelocity) annotation (Line(
              points={{-48,0},{-22,0}}, color={0,0,127}));
        connect(velocityFeedback, velocityPID.velocityFeedback) annotation (Line(
              points={{0,-80},{0,-12}}, color={0,0,127}));
        connect(velocityPID.setCurrent, currentPID.setCurrent) annotation (Line(
              points={{22,0},{68,0}}, color={0,0,127}));
        connect(currentFeedback, currentPID.currentFeedback) annotation (Line(
              points={{90,-80},{90,-12}}, color={0,0,127}));
        connect(currentPID.setPWM, P2V1.inputPWM) annotation (Line(points={{112,
                0},{128,0}}, color={0,0,127}));
        annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{
                  -200,-100},{200,100}}), graphics={
              Rectangle(extent={{-200,100},{200,-100}}, lineColor={0,0,255}),
              Text(
                extent={{-160,60},{140,0}},
                lineColor={0,0,255},
                textString=
                     "Position"),
              Text(
                extent={{-160,0},{140,-60}},
                lineColor={0,0,255},
                textString=
                     "Mode")}),
                              Diagram(coordinateSystem(preserveAspectRatio=
                  false, extent={{-200,-100},{200,100}}), graphics));
      end PositionMode;

      model VRampMode
        import youBot;
        import Modelica.SIunits;
        import Modelica.Blocks.Math;
        import Modelica.Blocks.Interfaces;

        Interfaces.RealInput positionFeedback
          annotation (Placement(transformation(
              origin={-140,-78},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        Interfaces.RealInput velocityFeedback
          annotation (Placement(transformation(
              origin={40,-80},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        Interfaces.RealInput currentFeedback
          annotation (Placement(transformation(
              origin={100,-80},
              extent={{-10,-10},{10,10}},
              rotation=90)));
        youBot.Controller.PIDs.VelocityPID velocityPID(
          PParam1=PParam1Velocity,
          PParam2=PParam2Velocity,
          IClip1=IClip1Velocity,
          IClip2=IClip2Velocity,
          IParam1=IParam1Velocity,
          IParam2=IParam2Velocity,
          DParam1=DParam1Velocity,
          DParam2=DParam2Velocity,
          threshold=thresholdVelocity,
          maximumCurrent=maximumCurrent,
          PIDPeriod=PIDPeriodVelocity)
                                 annotation (Placement(transformation(extent={{
                  20,-10},{60,10}}, rotation=0)));
        youBot.Controller.PIDs.CurrentPID currentPID(
          maximumPWM=maximumPWM,
          PIDPeriod=PIDPeriodCurrent,
          IClip=IClipCurrent,
          PParam=PParamCurrent,
          IParam=IParamCurrent,
          DParam=DParamCurrent)
                               annotation (Placement(transformation(extent={{80,
                  -10},{120,10}}, rotation=0)));
        Modelica.Blocks.Interfaces.RealOutput setVoltage
          annotation (Placement(transformation(extent={{170,-10},{190,10}},
                rotation=0)));
        youBot.Controller.Components.P2V P2V1(maximumVoltage=motorVoltage,
            maximumPWM=1799)
                 annotation (Placement(transformation(extent={{140,-10},{160,10}},
                rotation=0)));

      public
        parameter Integer encoderTickPerRound = 4000 "Motor Encoder Tick";
        parameter Real gearRatio = 156 "Gearbox Ratio";

        parameter SIunits.AngularVelocity thresholdPosition = 0
          "|Position Controller|| threshold velocity for position controller";
        parameter SIunits.AngularVelocity maximumVelocity = 2
          "|Position Controller|| maximum allowable velocity set";
        parameter SIunits.Time PIDPeriodPosition = 0.001
          "|Position Controller|| Time period for position pid controller";
        parameter Integer PParam1Position
          "|Position Controller|| first parameter for proportional gain in position controller";
        parameter Integer PParam2Position
          "|Position Controller|| second parameter for proportional gain in position controller";
        parameter Integer IParam1Position
          "|Position Controller|| first parameter for integral gain in position controller";
        parameter Integer IParam2Position
          "|Position Controller|| second parameter for integral gain in position controller";
        parameter Integer DParam1Position
          "|Position Controller|| first parameter for derivative gain in position controller";
        parameter Integer DParam2Position
          "|Position Controller|| second parameter for derivative gain in position controller";
        parameter Integer IClip1Position
          "|Position Controller|| first parameter for integral clip in position controller";
        parameter Integer IClip2Position
          "|Position Controller|| second parameter for integral clip in position controller";

        parameter SIunits.AngularAcceleration acceleration
          "|Velocity Ramp Generator|| Acceleration";
        parameter SIunits.Angle positionTolerance
          "|Velocity Ramp Generator|| Position tolerance";
        parameter SIunits.AngularVelocity velocityTolerance
          "|Velocity Ramp Generator|| Velocity tolerance";
        parameter SIunits.Angle stoppingDistanceAdjustment = 0.05
          "|Velocity Ramp Generator|| Stopping distance adjusment";
         parameter SIunits.Time startTime = 0.1
          "|Velocity Ramp Generator|| Time when Velocity Ramp Generator is active";

        parameter SIunits.AngularVelocity thresholdVelocity
          "|Velocity Controller|| threshold velocity for velocity controller";
        parameter Modelica.SIunits.Current maximumCurrent = 2.68
          "|Velocity Controller|| maximum allowable current set";
        parameter SIunits.Time PIDPeriodVelocity = 0.001
          "|Velocity Controller|| Time period for velocity pid controller";
        parameter Integer PParam1Velocity
          "|Velocity Controller|| first parameter for proportional gain in velocity controller";
        parameter Integer PParam2Velocity
          "|Velocity Controller|| second parameter for proportional gain in velocity controller";
        parameter Integer IParam1Velocity
          "|Velocity Controller|| first parameter for integral gain in velocity controller";
        parameter Integer IParam2Velocity
          "|Velocity Controller|| second parameter for integral gain in velocity controller";
        parameter Integer DParam1Velocity
          "|Velocity Controller|| first parameter for derivative gain in velocity controller";
        parameter Integer DParam2Velocity
          "|Velocity Controller|| second parameter for derivative gain in velocity controller";
        parameter Integer IClip1Velocity
          "|Velocity Controller|| first parameter for integral clip in velocity controller";
        parameter Integer IClip2Velocity
          "|Velocity Controller|| second parameter for integral clip in velocity controller";

        parameter Integer maximumPWM = 32767
          "|Current Controller|| maximum allowable PWM set";
        parameter SIunits.Time PIDPeriodCurrent = 0.001
          "|Current Controller|| Time period for current pid controller";
        parameter Integer PParamCurrent
          "|Current Controller|| proportional gain in current controller";
        parameter Integer IParamCurrent
          "|Current Controller|| integral gain in current controller";
        parameter Integer DParamCurrent
          "|Current Controller|| derivative gain in current controller";
        parameter Integer IClipCurrent = 0
          "|Current Controller|| integral clip in current controller";

        parameter SIunits.Voltage motorVoltage " Source voltage";

        youBot.Controller.Components.VelocityRampGenerator
          velocityRampGenerator(
          maximumVelocity=maximumVelocity,
          acceleration=acceleration,
          positionTolerance=positionTolerance,
          velocityTolerance=velocityTolerance,
          stoppingDistanceAdjustment=stoppingDistanceAdjustment,
          startTime=startTime)   annotation (Placement(transformation(extent={{
                  -110,20},{-90,40}}, rotation=0)));
        Interfaces.RealInput setPosition
          annotation (Placement(transformation(extent={{-190,-10},{-170,10}},
                rotation=0)));
        Math.Feedback feedback                  annotation (Placement(
              transformation(extent={{-150,-10},{-130,10}}, rotation=0)));
        youBot.Controller.PIDs.PositionPID positionPID(
          threshold=thresholdPosition,
          IClip1=IClip1Position,
          IClip2=IClip2Position,
          PParam1=PParam1Position,
          PParam2=PParam2Position,
          IParam1=IParam1Position,
          IParam2=IParam2Position,
          DParam1=DParam1Position,
          DParam2=DParam2Position,
          encoderTickPerRound=encoderTickPerRound,
          maximumVelocity=maximumVelocity,
          gearRatio=gearRatio,
          PIDPeriod=PIDPeriodPosition)
                                 annotation (Placement(transformation(extent={{
                  -80,-40},{-40,-20}}, rotation=0)));
        Modelica.Blocks.Logical.Switch switch
          annotation (Placement(transformation(extent={{-14,-10},{6,10}},
                rotation=0)));
        Modelica.Blocks.Sources.BooleanStep BooleanStep1(startTime=startTime,
            startValue=false) annotation (Placement(transformation(extent={{-80,
                  -10},{-60,10}}, rotation=0)));
      equation
        connect(setPosition, feedback.u1)     annotation (Line(points={{-180,0},
                {-148,0}}, color={0,0,127}));
        connect(positionFeedback, feedback.u2)   annotation (Line(points={{-140,
                -78},{-140,-8}}, color={0,0,127}));
        connect(P2V1.outputVoltage, setVoltage)     annotation (Line(points={{
                161,0},{180,0}}, color={0,0,127}));
        connect(velocityFeedback, positionPID.velocityFeedback)   annotation (Line(
              points={{40,-80},{40,-60},{-60,-60},{-60,-42}}, color={0,0,127}));
        connect(switch.y, velocityPID.setVelocity)
          annotation (Line(points={{7,0},{18,0}}, color={0,0,127}));
        connect(velocityFeedback, velocityPID.velocityFeedback) annotation (Line(
              points={{40,-80},{40,-12}}, color={0,0,127}));
        connect(feedback.y, positionPID.positionError) annotation (Line(points=
                {{-131,0},{-120,0},{-120,-30},{-82,-30}}, color={0,0,127}));
        connect(positionPID.setVelocity, switch.u3) annotation (Line(points={{
                -38,-30},{-30,-30},{-30,-8},{-16,-8}}, color={0,0,127}));
        connect(velocityPID.setCurrent, currentPID.setCurrent) annotation (Line(
              points={{62,0},{78,0}}, color={0,0,127}));
        connect(currentFeedback, currentPID.currentFeedback) annotation (Line(
              points={{100,-80},{100,-12}}, color={0,0,127}));
        connect(currentPID.setPWM, P2V1.inputPWM) annotation (Line(points={{122,
                0},{138,0}}, color={0,0,127}));
        connect(feedback.y, velocityRampGenerator.feedbackPosition) annotation (Line(
              points={{-131,0},{-120,0},{-120,30},{-112,30}}, color={0,0,127}));
        connect(velocityFeedback, velocityRampGenerator.feedbackVelocity)
          annotation (Line(points={{40,-80},{40,-60},{-100,-60},{-100,18}},
              color={0,0,127}));
        connect(velocityRampGenerator.setVelocity, switch.u1) annotation (Line(
              points={{-89,30},{-30,30},{-30,8},{-16,8}}, color={0,0,127}));
        connect(BooleanStep1.y, switch.u2) annotation (Line(points={{-59,0},{
                -16,0}}, color={255,0,255}));

        annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{
                  -200,-100},{200,100}}), graphics={
              Rectangle(extent={{-200,100},{200,-100}}, lineColor={0,0,255}),
              Text(
                extent={{-160,60},{140,0}},
                lineColor={0,0,255},
                textString=
                     "VRAmp"),
              Text(
                extent={{-160,0},{140,-60}},
                lineColor={0,0,255},
                textString=
                     "Mode")}),
                              Diagram(coordinateSystem(preserveAspectRatio=
                  false, extent={{-200,-100},{200,100}}), graphics));
      end VRampMode;
    end Modes;
  end Controller;

  package Axis

    model PositionAxis
      import Modelica.SIunits;
      Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet
        DCPM(
        IaNominal=nominalCurrent,
        Jr =    motorInertia + gearboxInertia,
        Ra=armatureResistance,
        La=armatureInductance,
        VaNominal=nominalVoltage,
        wNominal=(nominalRPM)*2*3.14159265358979323846/60)
             annotation (Placement(transformation(extent={{-30,-10},{-10,10}},
              rotation=0)));
      Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage
        annotation (Placement(transformation(
            origin={-20,48},
            extent={{-10,10},{10,-10}},
            rotation=180)));
      Modelica.Electrical.Analog.Basic.Ground ground
        annotation (Placement(transformation(extent={{10,20},{30,40}}, rotation=
               0)));
      Modelica.Mechanics.Rotational.Components.IdealGear
                                              idealGear(ratio=gearboxRatio,
          useSupport=false)
        annotation (Placement(transformation(extent={{20,-10},{40,10}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor
        annotation (Placement(transformation(extent={{20,-80},{0,-60}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor
        annotation (Placement(transformation(extent={{20,-60},{0,-40}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction
                                                    bearingFriction(tau_pos=[0,
            0.0328; 3,0.063; 4,0.0731], useSupport=false)
                                        annotation (Placement(transformation(
              extent={{60,-10},{80,10}}, rotation=0)));
      Modelica.Blocks.Interfaces.RealInput setPosition
        annotation (Placement(transformation(
            origin={-120,0},
            extent={{10,10},{-10,-10}},
            rotation=180)));
    public
      parameter SIunits.Voltage nominalVoltage = 24 "|Motor||";
      parameter SIunits.Current nominalCurrent = 2.36 "|Motor||";
      parameter SIunits.Conversions.NonSIunits.AngularVelocity_rpm nominalRPM =          6700
        "|Motor||";
      parameter SIunits.Resistance armatureResistance = 0.978 "|Motor||";
      parameter SIunits.Inductance armatureInductance = 0.000573 "|Motor||";
      parameter SIunits.Inertia motorInertia = 0.0000135 "|Motor||";
      parameter SIunits.Inertia gearboxInertia = 0.000000409 "|Motor||";
      parameter Real gearboxRatio = 100 "|Motor||";
      parameter Integer encoderTickPerRound = 4000 "|Motor||";

      parameter SIunits.AngularVelocity thresholdPosition
        "|Position Controller|| threshold velocity for position controller";
      parameter SIunits.AngularVelocity maximumVelocity = 2
        "|Position Controller|| maximum allowable velocity set";
      parameter SIunits.Time PIDPeriodPosition = 0.001
        "|Position Controller|| Time period for position pid controller";
      parameter Integer PParam1Position
        "|Position Controller|| first parameter for proportional gain in position controller";
      parameter Integer PParam2Position
        "|Position Controller|| second parameter for proportional gain in position controller";
      parameter Integer IParam1Position
        "|Position Controller|| first parameter for integral gain in position controller";
      parameter Integer IParam2Position
        "|Position Controller|| second parameter for integral gain in position controller";
      parameter Integer DParam1Position
        "|Position Controller|| first parameter for derivative gain in position controller";
      parameter Integer DParam2Position
        "|Position Controller|| second parameter for derivative gain in position controller";
      parameter Integer IClip1Position
        "|Position Controller|| first parameter for integral clip in position controller";
      parameter Integer IClip2Position
        "|Position Controller|| second parameter for integral clip in position controller";

      parameter SIunits.AngularVelocity thresholdVelocity
        "|Velocity Controller|| threshold velocity for velocity controller";
      parameter Modelica.SIunits.Current maximumCurrent = 2.68
        "|Velocity Controller|| maximum allowable current set";
      parameter SIunits.Time PIDPeriodVelocity = 0.001
        "|Velocity Controller|| Time period for velocity pid controller";
      parameter Integer PParam1Velocity
        "|Velocity Controller|| first parameter for proportional gain in velocity controller";
      parameter Integer PParam2Velocity
        "|Velocity Controller|| second parameter for proportional gain in velocity controller";
      parameter Integer IParam1Velocity
        "|Velocity Controller|| first parameter for integral gain in velocity controller";
      parameter Integer IParam2Velocity
        "|Velocity Controller|| second parameter for integral gain in velocity controller";
      parameter Integer DParam1Velocity
        "|Velocity Controller|| first parameter for derivative gain in velocity controller";
      parameter Integer DParam2Velocity
        "|Velocity Controller|| second parameter for derivative gain in velocity controller";
      parameter Integer IClip1Velocity
        "|Velocity Controller|| first parameter for integral clip in velocity controller";
      parameter Integer IClip2Velocity
        "|Velocity Controller|| second parameter for integral clip in velocity controller";

      parameter Integer maximumPWM = 32767
        "|Current Controller|| maximum allowable PWM set";
      parameter SIunits.Time PIDPeriodCurrent = 0.001
        "|Current Controller|| Time period for current pid controller";
      parameter Integer PParamCurrent
        "|Current Controller|| proportional gain in current controller";
      parameter Integer IParamCurrent
        "|Current Controller|| integral gain in current controller";
      parameter Integer DParamCurrent
        "|Current Controller|| derivative gain in current controller";
      parameter Integer IClipCurrent = 0
        "|Current Controller|| integral clip in current controller";

      Controller.Modes.PositionMode positionMode(
        motorVoltage=nominalVoltage,
        encoderTickPerRound=encoderTickPerRound,
        thresholdPosition=thresholdPosition,
        maximumVelocity=maximumVelocity,
        PIDPeriodPosition=PIDPeriodPosition,
        PParam1Position=PParam1Position,
        PParam2Position=PParam2Position,
        IParam1Position=IParam1Position,
        IParam2Position=IParam2Position,
        DParam1Position=DParam1Position,
        DParam2Position=DParam2Position,
        IClip1Position=IClip1Position,
        IClip2Position=IClip2Position,
        thresholdVelocity=thresholdVelocity,
        maximumCurrent=maximumCurrent,
        PIDPeriodVelocity=PIDPeriodVelocity,
        PParam1Velocity=PParam1Velocity,
        PParam2Velocity=PParam2Velocity,
        IParam1Velocity=IParam1Velocity,
        IParam2Velocity=IParam2Velocity,
        DParam1Velocity=DParam1Velocity,
        DParam2Velocity=DParam2Velocity,
        IClip1Velocity=IClip1Velocity,
        IClip2Velocity= IClip2Velocity,
        maximumPWM=maximumPWM,
        PIDPeriodCurrent=PIDPeriodCurrent,
        PParamCurrent=PParamCurrent,
        IParamCurrent=IParamCurrent,
        DParamCurrent=DParamCurrent,
        IClipCurrent=IClipCurrent,
        gearRatio=idealGear.ratio)
        annotation (Placement(transformation(extent={{-88,60},{-48,80}},
              rotation=0)));

      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b
        annotation (Placement(transformation(extent={{110,-10},{130,10}},
              rotation=0)));
    equation
      connect(ground.p, signalVoltage.p)
        annotation (Line(points={{20,40},{20,48},{-10,48}}, color={0,0,255}));
      connect(idealGear.flange_b, bearingFriction.flange_a)   annotation (Line(
            points={{40,0},{60,0}}, color={0,0,0}));
       positionMode.currentFeedback = DCPM.airGapDC.ia;
      connect(signalVoltage.n, DCPM.pin_an)  annotation (Line(points={{-30,48},
              {-40,48},{-40,10},{-26,10}}, color={0,0,255}));
      connect(signalVoltage.p, DCPM.pin_ap)  annotation (Line(points={{-10,48},
              {0,48},{0,10},{-14,10}}, color={0,0,255}));
      connect(bearingFriction.flange_b, flange_b)  annotation (Line(points={{80,
              0},{120,0}}, color={0,0,0}));
      connect(speedSensor.flange,   flange_b)  annotation (Line(points={{20,-50},
              {100,-50},{100,0},{120,0}}, color={0,0,0}));
      connect(angleSensor.flange,   flange_b)  annotation (Line(points={{20,-70},
              {100,-70},{100,0},{120,0}}, color={0,0,0}));
      connect(DCPM.flange,   idealGear.flange_a)
        annotation (Line(points={{-10,0},{20,0}}, color={0,0,0}));
      connect(setPosition, positionMode.setPosition) annotation (Line(points={{
              -120,0},{-100,0},{-100,70},{-86,70}}, color={0,0,127}));
      connect(speedSensor.w, positionMode.velocityFeedback) annotation (Line(
            points={{-1,-50},{-68,-50},{-68,62}}, color={0,0,127}));
      connect(angleSensor.phi, positionMode.positionFeedback) annotation (Line(
            points={{-1,-70},{-80,-70},{-80,62}}, color={0,0,127}));
      connect(positionMode.setVoltage, signalVoltage.v) annotation (Line(points=
             {{-50,70},{-20,70},{-20,55}}, color={0,0,127}));
      annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent={{
                -140,-100},{140,100}}), graphics={Line(
              points={{0,0},{-58,0},{-58,62}},
              color={0,0,255},
              pattern=LinePattern.Dot)}),
        Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-100},{
                140,100}}), graphics={
            Rectangle(
              extent={{-140,60},{140,-60}},
              lineColor={255,128,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid),
            Text(
              extent={{-100,40},{100,0}},
              lineColor={0,0,0},
              pattern=LinePattern.None,
              textString=
                   "Position"),
            Text(
              extent={{-102,-2},{98,-42}},
              lineColor={0,0,0},
              pattern=LinePattern.None,
              textString=
                   "Axis"),
            Text(
              extent={{-140,100},{140,60}},
              lineColor={0,0,255},
              textString=
                   "%name")}));
    end PositionAxis;

    model VRampAxis
      import Modelica.SIunits;
      import Modelica.SIunits.Conversions.NonSIunits;
      Modelica.Electrical.Machines.BasicMachines.DCMachines.DC_PermanentMagnet
        DCPM(
        IaNominal=nominalCurrent,
        Jr =    motorInertia + gearboxInertia,
        Ra=armatureResistance,
        La=armatureInductance,
        VaNominal=nominalVoltage,
        wNominal=(nominalRPM)*2*3.14159265358979323846/60)
             annotation (Placement(transformation(extent={{-30,-10},{-10,10}},
              rotation=0)));
      Modelica.Electrical.Analog.Sources.SignalVoltage signalVoltage
        annotation (Placement(transformation(
            origin={-20,48},
            extent={{-10,10},{10,-10}},
            rotation=180)));
      Modelica.Electrical.Analog.Basic.Ground ground
        annotation (Placement(transformation(extent={{10,20},{30,40}}, rotation=
               0)));
      Modelica.Mechanics.Rotational.Components.IdealGear
                                              idealGear(ratio=gearboxRatio,
          useSupport=false)
        annotation (Placement(transformation(extent={{20,-10},{40,10}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor
        annotation (Placement(transformation(extent={{20,-80},{0,-60}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor
        annotation (Placement(transformation(extent={{20,-60},{0,-40}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Components.BearingFriction
                                                    bearingFriction(tau_pos=[0,
            0.0328; 3,0.063; 4,0.0731], useSupport=false)
                                        annotation (Placement(transformation(
              extent={{60,-10},{80,10}}, rotation=0)));

    public
      parameter SIunits.Voltage nominalVoltage = 24 "|Motor||";
      parameter SIunits.Current nominalCurrent = 2.36 "|Motor||";
      parameter SIunits.Conversions.NonSIunits.AngularVelocity_rpm nominalRPM =          6700
        "|Motor||";
      parameter SIunits.Resistance armatureResistance = 0.978 "|Motor||";
      parameter SIunits.Inductance armatureInductance = 0.000573 "|Motor||";
      parameter SIunits.Inertia motorInertia = 0.0000135 "|Motor||";
      parameter SIunits.Inertia gearboxInertia = 0.000000409 "|Motor||";
      parameter Real gearboxRatio = 100 "|Motor||";
      parameter Integer encoderTickPerRound = 4000 "|Motor||";

      parameter SIunits.AngularAcceleration acceleration = 2
        "|Velocity Ramp Generator|| Acceleration";
      parameter Real positionTolerance = 5
        "|Velocity Ramp Generator|| Position tolerance, EncoderTick";
      parameter SIunits.AngularVelocity velocityTolerance
        "|Velocity Ramp Generator|| Velocity tolerance";
      parameter SIunits.Angle stoppingDistanceAdjustment = 0.05
        "|Velocity Ramp Generator|| Stopping distance adjusment";
       parameter SIunits.Time startTime = 0.1
        "|Velocity Ramp Generator|| Time when Velocity Ramp Generator is active";
      parameter SIunits.AngularVelocity thresholdPosition
        "|Position Controller|| threshold velocity for position controller";
      parameter SIunits.AngularVelocity maximumVelocity = 2
        "|Position Controller|| maximum allowable velocity set";
      parameter SIunits.Time PIDPeriodPosition = 0.001
        "|Position Controller|| Time period for position pid controller";
      parameter Integer PParam1Position
        "|Position Controller|| first parameter for proportional gain in position controller";
      parameter Integer PParam2Position
        "|Position Controller|| second parameter for proportional gain in position controller";
      parameter Integer IParam1Position
        "|Position Controller|| first parameter for integral gain in position controller";
      parameter Integer IParam2Position
        "|Position Controller|| second parameter for integral gain in position controller";
      parameter Integer DParam1Position
        "|Position Controller|| first parameter for derivative gain in position controller";
      parameter Integer DParam2Position
        "|Position Controller|| second parameter for derivative gain in position controller";
      parameter Integer IClip1Position
        "|Position Controller|| first parameter for integral clip in position controller";
      parameter Integer IClip2Position
        "|Position Controller|| second parameter for integral clip in position controller";

      parameter SIunits.AngularVelocity thresholdVelocity
        "|Velocity Controller|| threshold velocity for velocity controller";
      parameter SIunits.Current maximumCurrent = 2
        "|Velocity Controller|| (A), maximum allowable current set";
      parameter SIunits.Time PIDPeriodVelocity = 0.001
        "|Velocity Controller|| Time period for velocity pid controller";
      parameter Integer PParam1Velocity
        "|Velocity Controller|| first parameter for proportional gain in velocity controller";
      parameter Integer PParam2Velocity
        "|Velocity Controller|| second parameter for proportional gain in velocity controller";
      parameter Integer IParam1Velocity
        "|Velocity Controller|| first parameter for integral gain in velocity controller";
      parameter Integer IParam2Velocity
        "|Velocity Controller|| second parameter for integral gain in velocity controller";
      parameter Integer DParam1Velocity
        "|Velocity Controller|| first parameter for derivative gain in velocity controller";
      parameter Integer DParam2Velocity
        "|Velocity Controller|| second parameter for derivative gain in velocity controller";
      parameter Integer IClip1Velocity
        "|Velocity Controller|| first parameter for integral clip in velocity controller";
      parameter Integer IClip2Velocity
        "|Velocity Controller|| second parameter for integral clip in velocity controller";

      parameter Integer maximumPWM
        "|Current Controller|| maximum allowable PWM set";
      parameter SIunits.Time PIDPeriodCurrent = 0.001
        "|Current Controller|| Time period for current pid controller";
      parameter Integer PParamCurrent
        "|Current Controller|| proportional gain in current controller";
      parameter Integer IParamCurrent
        "|Current Controller|| integral gain in current controller";
      parameter Integer DParamCurrent
        "|Current Controller|| derivative gain in current controller";
      parameter Integer IClipCurrent = 0
        "|Current Controller|| integral clip in current controller";

      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b
        annotation (Placement(transformation(extent={{110,-10},{130,10}},
              rotation=0)));
      Controller.Modes.VRampMode Controller_VRampMode2_1(
        motorVoltage=nominalVoltage,
        maximumVelocity=maximumVelocity,
        thresholdVelocity=thresholdVelocity,
        maximumCurrent=maximumCurrent,
        PIDPeriodVelocity=PIDPeriodVelocity,
        PParam1Velocity=PParam1Velocity,
        PParam2Velocity=PParam2Velocity,
        IParam1Velocity=IParam1Velocity,
        IParam2Velocity=IParam2Velocity,
        DParam1Velocity=DParam1Velocity,
        DParam2Velocity=DParam2Velocity,
        IClip1Velocity=IClip1Velocity,
        IClip2Velocity=IClip2Velocity,
        velocityTolerance=velocityTolerance,
        PIDPeriodCurrent=PIDPeriodCurrent,
        PParamCurrent=PParamCurrent,
        IParamCurrent=IParamCurrent,
        DParamCurrent=DParamCurrent,
        IClipCurrent=IClipCurrent,
        acceleration=acceleration,
        stoppingDistanceAdjustment=stoppingDistanceAdjustment,
        PIDPeriodPosition=PIDPeriodPosition,
        PParam1Position=PParam1Position,
        PParam2Position=PParam2Position,
        IParam1Position=IParam1Position,
        IParam2Position=IParam2Position,
        DParam1Position=DParam1Position,
        DParam2Position=DParam2Position,
        IClip1Position=IClip1Position,
        IClip2Position=IClip2Position,
        thresholdPosition=thresholdPosition,
        startTime=1,
        positionTolerance=positionTolerance/encoderTickPerRound*(65535/
            encoderTickPerRound)*2/idealGear.ratio*Modelica.Constants.pi)
        annotation (Placement(transformation(extent={{-90,60},{-48,80}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput setPosition
        annotation (Placement(transformation(
            origin={-120,0},
            extent={{10,10},{-10,-10}},
            rotation=180)));
    equation
      connect(ground.p, signalVoltage.p)
        annotation (Line(points={{20,40},{20,48},{-10,48}}, color={0,0,255}));
      connect(idealGear.flange_b, bearingFriction.flange_a)   annotation (Line(
            points={{40,0},{60,0}}, color={0,0,0}));
      Controller_VRampMode2_1.currentFeedback = DCPM.airGapDC.ia;
      connect(signalVoltage.n, DCPM.pin_an)  annotation (Line(points={{-30,48},
              {-40,48},{-40,10},{-26,10}}, color={0,0,255}));
      connect(signalVoltage.p, DCPM.pin_ap)  annotation (Line(points={{-10,48},
              {0,48},{0,10},{-14,10}}, color={0,0,255}));
      connect(speedSensor.flange,   flange_b)  annotation (Line(points={{20,-50},
              {100,-50},{100,0},{120,0}}, color={0,0,0}));
      connect(bearingFriction.flange_b, flange_b)
        annotation (Line(points={{80,0},{120,0}}, color={0,0,0}));
      connect(angleSensor.flange,   flange_b)  annotation (Line(points={{20,-70},
              {100,-70},{100,0},{120,0}}, color={0,0,0}));
      connect(DCPM.flange,   idealGear.flange_a)
        annotation (Line(points={{-10,0},{20,0}}, color={0,0,0}));
      connect(angleSensor.phi, Controller_VRampMode2_1.positionFeedback)
        annotation (Line(points={{-1,-70},{-83.7,-70},{-83.7,62.2}}, color={0,0,
              127}));
      connect(Controller_VRampMode2_1.setVoltage, signalVoltage.v)  annotation (Line(
            points={{-50.1,70},{-20,70},{-20,55}}, color={0,0,127}));
      connect(setPosition, Controller_VRampMode2_1.setPosition)  annotation (Line(
            points={{-120,0},{-100,0},{-100,70},{-87.9,70}}, color={0,0,127}));
      connect(speedSensor.w, Controller_VRampMode2_1.velocityFeedback)
        annotation (Line(points={{-1,-50},{-64.8,-50},{-64.8,62}}, color={0,0,
              127}));
      annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent={{
                -140,-100},{140,100}}), graphics={Line(
              points={{-58,62},{-58,0},{-30,0}},
              color={0,0,255},
              pattern=LinePattern.Dot)}),
        Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-100},{
                140,100}}), graphics={
            Rectangle(
              extent={{-140,60},{140,-60}},
              lineColor={255,128,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid),
            Text(
              extent={{-100,40},{100,0}},
              lineColor={0,0,0},
              pattern=LinePattern.None,
              textString=
                   "VRamp"),
            Text(
              extent={{-102,-2},{98,-42}},
              lineColor={0,0,0},
              pattern=LinePattern.None,
              textString=
                   "Axis"),
            Text(
              extent={{-140,100},{140,60}},
              lineColor={0,0,255},
              textString=
                   "%name")}));
    end VRampAxis;
  end Axis;

  package Body
    model Link
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a
        annotation (Placement(transformation(extent={{-120,-10},{-100,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b
        annotation (Placement(transformation(extent={{100,-10},{120,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frameOfReference(
        color_x={255,0,0},
        color_y={0,180,0},
        color_z={0,0,255},
        showLabels=true,
        length=0.2,
        animation=jointFrameOfReference)
                        annotation (Placement(transformation(extent={{46,-50},{
                66,-30}}, rotation=0)));
      Modelica.Mechanics.MultiBody.Joints.Revolute         joint(animation=
            false,
        useAxisFlange=true,
        a(fixed=false),
        phi(fixed=false, start=(angleInitial)*(3.14159265358979323846/180)),
        w(fixed=false))
        annotation (Placement(transformation(extent={{-10,-10},{10,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.FixedRotation frameRotation(animation=
            false)
        annotation (Placement(transformation(extent={{-60,-10},{-40,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.BodyShape body(animateSphere=false,
          animation=bodyAnimation)
        annotation (Placement(transformation(extent={{40,-10},{60,10}},
              rotation=0)));
    public
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg angleInitial=0;
      parameter Boolean jointFrameOfReference = false
        "Frame of reference for joint";
      parameter Boolean bodyAnimation = true "Show the body";
      Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a
        annotation (Placement(transformation(extent={{-10,40},{10,60}},
              rotation=0)));
    equation
      connect(frame_a,frameRotation. frame_a)  annotation (Line(
          points={{-110,0},{-61,0}},
          color={0,0,0},
          thickness=0.5));
      connect(frameRotation.frame_b,joint. frame_a)              annotation (Line(
          points={{-39,0},{-11,0}},
          color={0,0,0},
          thickness=0.5));
      connect(joint.frame_b,body. frame_a)                   annotation (Line(
          points={{11,0},{39,0}},
          color={0,0,0},
          thickness=0.5));
      connect(body.frame_b, frame_b)       annotation (Line(
          points={{61,0},{110,0}},
          color={0,0,0},
          thickness=0.5));
      connect(joint.axis, flange_a)
        annotation (Line(points={{0,10},{0,50}}, color={0,0,0}));
      connect(joint.frame_b,frameOfReference. frame_a) annotation (Line(
          points={{11,0},{20,0},{20,-40},{45,-40}},
          color={0,0,0},
          thickness=0.5));
      annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent={{
                -100,-80},{100,80}}), graphics),
                           Icon(coordinateSystem(preserveAspectRatio=false,
              extent={{-100,-80},{100,80}}), graphics={Rectangle(
              extent={{-100,40},{100,-40}},
              lineColor={0,0,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid), Text(
              extent={{-80,-40},{80,-100}},
              lineColor={0,0,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid,
              textString=
                   "%name")}));
    end Link;

    model Gripper
    public
      parameter Boolean bodyAnimation = true "show manipulator animation";
      parameter Boolean endPointFrameOfReference = false
        "Frame of reference for manipulator end point";
      parameter Boolean endPointAnimation = true
        "show manipulator end point animation";
      Modelica.Mechanics.MultiBody.Interfaces.Frame_a frame_a
        annotation (Placement(transformation(extent={{-120,-10},{-100,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.BodyShape gripperBaseBody(
        shapeType="7",
        r={0,0,0},
        m=0.199,
        I_11=0.0002324,
        I_22=0.0003629,
        I_33=0.0002067,
        color={0,0,0},
        animation=bodyAnimation)
        annotation (Placement(transformation(extent={{-40,-10},{-20,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.FixedRotation fixedRotation(
        rotationType=Modelica.Mechanics.MultiBody.Types.RotationTypes.
            PlanarRotationSequence,
        sequence={1,2,3},
        angles={0,0,180})
        annotation (Placement(transformation(extent={{-80,-10},{-60,10}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r={0,
            -0.0082,0})
        annotation (Placement(transformation(extent={{0,50},{20,70}}, rotation=
                0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation2(r={0,
            0.0082,0})
        annotation (Placement(transformation(extent={{2,-70},{22,-50}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.BodyShape leftFingerBody(
        lengthDirection={1,0,0},
        widthDirection={0,1,0},
        m=0.010,
        animateSphere=false,
        shapeType="8",
        color={131,131,131},
        animation=bodyAnimation)
        annotation (Placement(transformation(extent={{40,50},{60,70}}, rotation=
               0)));
      Modelica.Mechanics.MultiBody.Parts.BodyShape rightFingerBody(
        animateSphere=false,
        m=0.01,
        shapeType="9",
        lengthDirection={1,0,0},
        widthDirection={0,1,0},
        color={131,131,131},
        animation=bodyAnimation)
        annotation (Placement(transformation(extent={{40,-70},{60,-50}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation3(r={
            0,0,0.035}, animation=false)
                        annotation (Placement(transformation(extent={{0,-10},{
                20,10}}, rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.Body endPointMarker(
        m=0,
        sphereDiameter=0.01,
        animation=endPointAnimation)
                             annotation (Placement(transformation(extent={{42,
                -10},{62,10}}, rotation=0)));
      Modelica.Mechanics.MultiBody.Visualizers.FixedFrame frameOfReference(
        color_x={255,0,0},
        color_y={0,180,0},
        color_z={0,0,255},
        showLabels=true,
        length=0.2,
        animation=endPointFrameOfReference)
                        annotation (Placement(transformation(extent={{40,-40},{
                60,-20}}, rotation=0)));
    equation
      connect(frame_a, fixedRotation.frame_a)  annotation (Line(
          points={{-110,0},{-81,0}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedRotation.frame_b, gripperBaseBody.frame_a)
                                                          annotation (Line(
          points={{-59,0},{-41,0}},
          color={0,0,0},
          thickness=0.5));
      connect(gripperBaseBody.frame_b,fixedTranslation1. frame_a)
                                                             annotation (Line(
          points={{-19,0},{-10,0},{-10,60},{-1,60}},
          color={0,0,0},
          thickness=0.5));
      connect(gripperBaseBody.frame_b,fixedTranslation2. frame_a)
                                                             annotation (Line(
          points={{-19,0},{-10,0},{-10,-60},{1,-60}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedTranslation1.frame_b, leftFingerBody.frame_a)
                                                             annotation (Line(
          points={{21,60},{39,60}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedTranslation2.frame_b, rightFingerBody.frame_a)
                                                             annotation (Line(
          points={{23,-60},{39,-60}},
          color={0,0,0},
          thickness=0.5));
      connect(gripperBaseBody.frame_b,fixedTranslation3. frame_a) annotation (Line(
          points={{-19,0},{-1,0}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedTranslation3.frame_b, endPointMarker.frame_a)   annotation (Line(
          points={{21,0},{41,0}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedTranslation3.frame_b, frameOfReference.frame_a)  annotation (Line(
          points={{21,0},{30,0},{30,-30},{39,-30}},
          color={0,0,0},
          thickness=0.5));
      annotation (Diagram(coordinateSystem(preserveAspectRatio=false, extent={{
                -100,-80},{100,80}}), graphics),
                           Icon(coordinateSystem(preserveAspectRatio=false,
              extent={{-100,-80},{100,80}}), graphics={Rectangle(
              extent={{-100,40},{60,-40}},
              lineColor={0,0,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid), Text(
              extent={{-80,-40},{40,-80}},
              lineColor={0,0,0},
              fillColor={255,128,0},
              fillPattern=FillPattern.Solid,
              textString=
                   "gripper")}));
    end Gripper;

    model Manipulator
      import Modelica.SIunits;
      extends Modelica.Blocks.Interfaces.BlockIcon;
      parameter SIunits.Length baseX = 0
        "location of base_manipulator in x axis";
      parameter SIunits.Length baseY = 0
        "location of base_manipulator in y axis";
      parameter SIunits.Conversions.NonSIunits.Angle_deg joint5InitialAngle= 0
        "Initial angle for joint 5 at start of simulation";
      parameter SIunits.Conversions.NonSIunits.Angle_deg joint4InitialAngle= 0
        "Initial angle for joint 4 at start of simulation";
      parameter SIunits.Conversions.NonSIunits.Angle_deg joint3InitialAngle= 0
        "Initial angle for joint 3 at start of simulation";
      parameter SIunits.Conversions.NonSIunits.Angle_deg joint2InitialAngle= 0
        "Initial angle for joint 2 at start of simulation";
      parameter SIunits.Conversions.NonSIunits.Angle_deg joint1InitialAngle= 0
        "Initial angle for joint 1 at start of simulation";
      parameter Boolean manipulatorAnimation = true
        "show manipulator animation";
      parameter Boolean jointFrameOfReference = false
        "show frame of reference for each joint";
      parameter Boolean endPointFrameOfReference = false
        "Frame of reference for manipulator end point";
      parameter Boolean endPointAnimation = true
        "show manipulator end point animation";
      parameter Boolean worldAnimation = true
        "show world frame of reference animation";
      Link link1(
        frameRotation(
          rotationType=Modelica.Mechanics.MultiBody.Types.RotationTypes.
              PlanarRotationSequence,
          sequence={1,2,3},
          angles={180,0,0}),
        body(
          m=1.390,
          lengthDirection={1,0,0},
          widthDirection={0,1,0},
          shapeType="2",
          color={255,128,0},
          r={0.033,0,0},
          I_11=0.0029525,
          I_22=0.0060091,
          I_33=0.0058821),
        joint(animation=false),
        angleInitial=joint1InitialAngle,
        jointFrameOfReference=jointFrameOfReference,
        bodyAnimation=manipulatorAnimation)
                 annotation (Placement(transformation(
            origin={-36,-60},
            extent={{-10,-10},{10,10}},
            rotation=90)));
      Link link2(
        frameRotation(
          rotationType=Modelica.Mechanics.MultiBody.Types.RotationTypes.
              PlanarRotationSequence,
          sequence={1,2,3},
          angles={90,0,-90}),
        body(
          r={0.155,0,0},
          m=1.318,
          shapeType="3",
          lengthDirection={1,0,0},
          widthDirection={0,1,0},
          color={255,128,0},
          I_11=0.0031145,
          I_22=0.0005843,
          I_33=0.0031631),
        joint(animation=false, n={0,0,1}),
        angleInitial=joint2InitialAngle,
        jointFrameOfReference=jointFrameOfReference,
        bodyAnimation=manipulatorAnimation)
                 annotation (Placement(transformation(
            origin={-36,-30},
            extent={{-10,-10},{10,10}},
            rotation=90)));
      Link link3(
        frameRotation(
          rotationType=Modelica.Mechanics.MultiBody.Types.RotationTypes.
              PlanarRotationSequence,
          sequence={1,2,3},
          angles={0,0,-90}),
        body(
          r={0,0.135,0},
          m=0.821,
          shapeType="4",
          lengthDirection={1,0,0},
          widthDirection={0,1,0},
          color={255,128,0},
          I_11=0.00172767,
          I_22=0.00041967,
          I_33=0.0018468),
        joint(animation=false, n={0,0,1}),
        angleInitial=joint3InitialAngle,
        jointFrameOfReference=jointFrameOfReference,
        bodyAnimation=manipulatorAnimation)
                 annotation (Placement(transformation(
            origin={-36,0},
            extent={{-10,-10},{10,10}},
            rotation=90)));
      Link link4(
        body(
          r={0,0.11316,0},
          m=0.769,
          shapeType="5",
          lengthDirection={1,0,0},
          widthDirection={0,1,0},
          color={255,128,0},
          I_11=0.0006764,
          I_22=0.0010573,
          I_33=0.000661,
          animateSphere=false),
        joint(animation=false),
        angleInitial=joint4InitialAngle,
        jointFrameOfReference=jointFrameOfReference,
        bodyAnimation=manipulatorAnimation)
                 annotation (Placement(transformation(
            origin={-36,30},
            extent={{-10,-10},{10,10}},
            rotation=90)));
      Link link5(
        body(
          r={0,0,0.05716},
          m=0.687,
          lengthDirection={1,0,0},
          widthDirection={0,1,0},
          color={255,128,0},
          shapeType="6",
          I_11=0.0001934,
          I_22=0.0001602,
          I_33=0.0000689,
          animateSphere=false),
        frameRotation(
          rotationType=Modelica.Mechanics.MultiBody.Types.RotationTypes.
              PlanarRotationSequence,
          sequence={1,2,3},
          angles={-90,0,0}),
        joint(animation=false),
        angleInitial=joint5InitialAngle,
        jointFrameOfReference=jointFrameOfReference,
        bodyAnimation=manipulatorAnimation)
                 annotation (Placement(transformation(
            origin={-36,60},
            extent={{-10,-10},{10,10}},
            rotation=90)));
      Gripper gripper(
        bodyAnimation=manipulatorAnimation,
        endPointFrameOfReference=endPointFrameOfReference,
        endPointAnimation=endPointAnimation)
                       annotation (Placement(transformation(extent={{-12,70},{8,
                90}}, rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.BodyShape armBase(
        animateSphere=false,
        r={0.024,0,0.115},
        shapeType="1",
        m=0.961,
        color={0,0,0},
        lengthDirection={1,0,0},
        widthDirection={0,1,0},
        animation=manipulatorAnimation)
                 annotation (Placement(transformation(extent={{10,-90},{-10,-70}},
              rotation=0)));
      inner Modelica.Mechanics.MultiBody.World world(n={0,0,-1},
        axisDiameter=0.0075,
        axisLength=0.4,
        animateGravity=false,
        animateWorld=worldAnimation)
        annotation (Placement(transformation(extent={{88,-90},{68,-70}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a joint5
        annotation (Placement(transformation(extent={{-86,54},{-74,66}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a joint1
        annotation (Placement(transformation(extent={{-86,-66},{-74,-54}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a joint2
        annotation (Placement(transformation(extent={{-86,-36},{-74,-24}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a joint3
        annotation (Placement(transformation(extent={{-86,-6},{-74,6}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Interfaces.Flange_a joint4
        annotation (Placement(transformation(extent={{-86,24},{-74,36}},
              rotation=0)));
      Modelica.Mechanics.MultiBody.Parts.FixedTranslation fixedTranslation1(r={
            baseX,baseY,0.03}, animation=false)
        annotation (Placement(transformation(extent={{50,-90},{30,-70}},
              rotation=0)));
    equation
      connect(link1.frame_b, link2.frame_a) annotation (Line(
          points={{-36,-46.25},{-36,-43.75}},
          color={0,0,0},
          thickness=0.5));
      connect(link2.frame_b, link3.frame_a) annotation (Line(
          points={{-36,-16.25},{-36,-13.75}},
          color={0,0,0},
          thickness=0.5));
      connect(link3.frame_b, link4.frame_a) annotation (Line(
          points={{-36,13.75},{-36,16.25}},
          color={0,0,0},
          thickness=0.5));
      connect(link4.frame_b, link5.frame_a) annotation (Line(
          points={{-36,43.75},{-36,46.25}},
          color={0,0,0},
          thickness=0.5));
      connect(link5.frame_b, gripper.frame_a)  annotation (Line(
          points={{-36,73.75},{-36,80},{-13,80}},
          color={0,0,0},
          thickness=0.5));
      connect(armBase.frame_b, link1.frame_a)  annotation (Line(
          points={{-11,-80},{-36,-80},{-36,-73.75}},
          color={0,0,0},
          thickness=0.5));
      connect(link5.flange_a, joint5) annotation (Line(points={{-41,60},{-80,60}},
            color={0,0,0}));
      connect(joint1, link1.flange_a) annotation (Line(points={{-80,-60},{-41,
              -60}}, color={0,0,0}));
      connect(joint3, link3.flange_a) annotation (Line(points={{-80,0},{-58,0},
              {-58,3.82689e-016},{-41,3.82689e-016}}, color={0,0,0}));
      connect(joint2, link2.flange_a) annotation (Line(points={{-80,-30},{-41,
              -30}}, color={0,0,0}));
      connect(joint4, link4.flange_a) annotation (Line(points={{-80,30},{-41,30}},
            color={0,0,0}));
      connect(fixedTranslation1.frame_b, armBase.frame_a)  annotation (Line(
          points={{29,-80},{11,-80}},
          color={0,0,0},
          thickness=0.5));
      connect(fixedTranslation1.frame_a, world.frame_b) annotation (Line(
          points={{51,-80},{67,-80}},
          color={0,0,0},
          thickness=0.5));
      annotation (Diagram(coordinateSystem(
            preserveAspectRatio=false,
            extent={{-100,-100},{100,100}},
            grid={2,2}), graphics),
    Icon(coordinateSystem(
            preserveAspectRatio=false,
            extent={{-100,-100},{100,100}},
            grid={2,2}), graphics={Bitmap(extent={{-86,92},{94,-94}}, fileName=
                                              "body_icon.png")}));
    end Manipulator;

  end Body;

  package System
    model ManipulatorPositionMode
      import Modelica.SIunits;
      parameter SIunits.Length baseX = 0
        "| Initial Pose | Location | location of base_manipulator in world x axis";
      parameter SIunits.Length baseY = 0
        "| Initial Pose | Location | location of base_manipulator in world y axis";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint5InitialAngle = -165 + 75
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint4InitialAngle = -102.5 + 192.5
        "| Initial Pose | Initial Angles | Initial angle for joint 4 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint3InitialAngle = 146 - 236
        "| Initial Pose | Initial Angles | Initial angle for joint 3 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint2InitialAngle = -65 + 155
        "| Initial Pose | Initial Angles | Initial angle for joint 2 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint1InitialAngle = -169 + 79
        "| Initial Pose | Initial Angles | Initial angle for joint 1 at start of simulation";
      parameter Boolean manipulatorAnimation = true
        "| Animation | Animation | show manipulator body animation";
      parameter Boolean jointFrameOfReference = false
        "| Animation | Animation | show frame of reference for each joint";
      parameter Boolean endPointFrameOfReference = false
        "| Animation | Animation | show Frame of reference for manipulator end point";
      parameter Boolean endPointAnimation = true
        "| Animation | Animation | show manipulator end point animation";
      parameter Boolean worldAnimation = true
        "| Animation | Animation | show world frame of reference animation";

      Body.Manipulator youBotManipulatorBody(
        joint5InitialAngle=joint5InitialAngle,
        joint4InitialAngle=joint4InitialAngle,
        joint3InitialAngle=joint3InitialAngle,
        joint2InitialAngle=joint2InitialAngle,
        joint1InitialAngle=joint1InitialAngle,
        manipulatorAnimation=manipulatorAnimation,
        jointFrameOfReference=jointFrameOfReference,
        endPointFrameOfReference=endPointFrameOfReference,
        endPointAnimation=endPointAnimation,
        baseX=baseX,
        baseY=baseY,
        worldAnimation=worldAnimation)
                                  annotation (Placement(transformation(extent={
                {40,-20},{80,20}}, rotation=0)));
      Axis.PositionAxis positionAxis5(
        nominalVoltage=24,
        nominalCurrent=0.49,
        nominalRPM=2800,
        armatureResistance=13.7,
        armatureInductance=0.00773,
        motorInertia=0.0000035,
        gearboxInertia=0.000000071,
        encoderTickPerRound=4000,
        thresholdPosition=0,
        PIDPeriodPosition=0.001,
        IParam1Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        thresholdVelocity=0,
        maximumCurrent=1.75,
        PParam1Velocity=200,
        IParam1Velocity=200,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        PIDPeriodCurrent=0.001,
        PParamCurrent=4000,
        IParamCurrent=4000,
        DParamCurrent=0,
        IClipCurrent=1000,
        maximumVelocity=5.8997,
        PParam1Position=50,
        PParam2Position=50,
        PParam2Velocity=800,
        IParam2Velocity=600,
        maximumPWM=32767,
        IParam2Position=0,
        gearboxRatio=71)            annotation (Placement(transformation(extent=
               {{-40,50},{-20,70}}, rotation=0)));
      Axis.PositionAxis positionAxis4(
        nominalVoltage=24,
        nominalCurrent=1.07,
        nominalRPM=2850,
        armatureResistance=4.48,
        armatureInductance=0.00224,
        motorInertia=0.00000925,
        gearboxInertia=0.000000071,
        thresholdPosition=0,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        thresholdVelocity=0,
        maximumCurrent=2.748,
        PIDPeriodVelocity=0.001,
        PParam1Velocity=200,
        PParam2Velocity=1000,
        IParam1Velocity=200,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        PIDPeriodCurrent=0.001,
        DParamCurrent=0,
        IClipCurrent=1000,
        maximumVelocity=5.8997,
        PParam1Position=200,
        PParam2Position=200,
        IParam2Velocity=2000,
        maximumPWM=32767,
        PParamCurrent=3000,
        IParamCurrent=3000,
        encoderTickPerRound=4000,
        gearboxRatio=71)            annotation (Placement(transformation(extent=
               {{-40,20},{-20,40}}, rotation=0)));
      Axis.PositionAxis positionAxis3(
        nominalVoltage=24,
        nominalCurrent=0.236,
        nominalRPM=5260,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000071,
        encoderTickPerRound=4000,
        thresholdPosition=0,
        PIDPeriodPosition=0.001,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        thresholdVelocity=0,
        PIDPeriodVelocity=0.001,
        PParam1Velocity=200,
        IParam1Velocity=200,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        PIDPeriodCurrent=0.001,
        DParamCurrent=0,
        IClipCurrent=1000,
        maximumVelocity=4.8879,
        PParam1Position=200,
        PParam2Position=200,
        maximumCurrent=3.899,
        PParam2Velocity=2000,
        IParam2Velocity=4000,
        maximumPWM=32767,
        PParamCurrent=1500,
        IParamCurrent=1500,
        gearboxRatio=100)      annotation (Placement(transformation(extent={{
                -40,-10},{-20,10}}, rotation=0)));
      Axis.PositionAxis positionAxis2(
        nominalVoltage=24,
        nominalCurrent=2.36,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000409,
        encoderTickPerRound=4000,
        thresholdPosition=0,
        PIDPeriodPosition=0.001,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        thresholdVelocity=0,
        maximumCurrent=5.678,
        PIDPeriodVelocity=0.001,
        PParam1Velocity=200,
        IParam1Velocity=200,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        PIDPeriodCurrent=0.001,
        DParamCurrent=0,
        IClipCurrent=1000,
        nominalRPM=5260,
        maximumVelocity=2.68512,
        PParam1Position=200,
        PParam2Position=200,
        PParam2Velocity=2000,
        IParam2Velocity=6000,
        maximumPWM=32767,
        PParamCurrent=1500,
        IParamCurrent=1500,
        gearboxRatio=156)      annotation (Placement(transformation(extent={{
                -40,-40},{-20,-20}}, rotation=0)));
      Axis.PositionAxis positionAxis1(
        nominalVoltage=24,
        nominalCurrent=2.36,
        nominalRPM=6700,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000409,
        encoderTickPerRound=4000,
        thresholdPosition=0,
        PIDPeriodPosition=0.001,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        thresholdVelocity=0,
        maximumCurrent=5.768,
        PIDPeriodVelocity=0.001,
        PParam1Velocity=200,
        IParam1Velocity=200,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        PIDPeriodCurrent=0.001,
        DParamCurrent=0,
        IClipCurrent=1000,
        maximumVelocity=2.68512,
        PParam1Position=200,
        PParam2Position=200,
        PParam2Velocity=2500,
        IParam2Velocity=2500,
        maximumPWM=32767,
        PParamCurrent=1200,
        IParamCurrent=1000,
        gearboxRatio=156)      annotation (Placement(transformation(extent={{
                -40,-70},{-20,-50}}, rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint5SetPosition
        annotation (Placement(transformation(extent={{-90,50},{-70,70}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint4SetPosition
        annotation (Placement(transformation(extent={{-90,20},{-70,40}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint3SetPosition
        annotation (Placement(transformation(extent={{-90,-10},{-70,10}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint2SetPosition
        annotation (Placement(transformation(extent={{-90,-40},{-70,-20}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint1SetPosition
        annotation (Placement(transformation(extent={{-90,-70},{-70,-50}},
              rotation=0)));
    equation
      connect(positionAxis5.flange_b, youBotManipulatorBody.joint5)
                                                  annotation (Line(points={{
              -21.4286,60},{20,60},{20,12},{44,12}}, color={0,0,0}));
      connect(positionAxis4.flange_b, youBotManipulatorBody.joint4)
                                                  annotation (Line(points={{
              -21.4286,30},{10,30},{10,6},{44,6}}, color={0,0,0}));
      connect(positionAxis3.flange_b, youBotManipulatorBody.joint3)
                                                  annotation (Line(points={{
              -21.4286,0},{44,0}}, color={0,0,0}));
      connect(positionAxis2.flange_b, youBotManipulatorBody.joint2)
                                                  annotation (Line(points={{
              -21.4286,-30},{10,-30},{10,-6},{44,-6}}, color={0,0,0}));
      connect(positionAxis1.flange_b, youBotManipulatorBody.joint1)
                                                  annotation (Line(points={{
              -21.4286,-60},{20,-60},{20,-12},{44,-12}}, color={0,0,0}));
      connect(joint5SetPosition, positionAxis5.setPosition)
                                                    annotation (Line(points={{
              -80,60},{-38.5714,60}}, color={0,0,127}));
      connect(joint4SetPosition, positionAxis4.setPosition)
                                                    annotation (Line(points={{
              -80,30},{-38.5714,30}}, color={0,0,127}));
      connect(joint3SetPosition, positionAxis3.setPosition)
                                                    annotation (Line(points={{
              -80,0},{-38.5714,0}}, color={0,0,127}));
      connect(joint2SetPosition, positionAxis2.setPosition)
                                                    annotation (Line(points={{
              -80,-30},{-38.5714,-30}}, color={0,0,127}));
      connect(joint1SetPosition, positionAxis1.setPosition)
                                                    annotation (Line(points={{
              -80,-60},{-38.5714,-60}}, color={0,0,127}));
      annotation (Diagram(graphics),
                           Icon(graphics={
            Rectangle(
              extent={{-100,100},{100,-98}},
              lineColor={0,0,255},
              fillColor={255,255,255},
              fillPattern=FillPattern.Solid),
            Bitmap(extent={{-80,80},{80,-60}}, fileName=
                                                "icon_complete.png"),
            Text(
              extent={{-60,-60},{80,-100}},
              lineColor={0,0,255},
              textString=
                   "Position Mode")}));
    end ManipulatorPositionMode;

    model ManipulatorVRampMode
      import Modelica.SIunits;
      parameter SIunits.Length baseX = 0
        "| Initial Pose | Location | location of base_manipulator in x axis";
      parameter SIunits.Length baseY = 0
        "| Initial Pose | Location | location of base_manipulator in y axis";
      parameter Boolean manipulatorAnimation = true
        "| Animation | Animation | show manipulator animation";
      parameter Boolean jointFrameOfReference = false
        "| Animation | Animation | show frame of reference for each joint";
      parameter Boolean endPointFrameOfReference = false
        "| Animation | Animation | Frame of reference for manipulator end point";
      parameter Boolean endPointAnimation = true
        "| Animation | Animation | show manipulator end point animation";
      parameter Boolean worldAnimation = true
        "| Animation | Animation | show world frame of reference animation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint1InitialAngle = -169 + 79
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint2InitialAngle = -65 + 155
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint3InitialAngle = 146 - 236
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint4InitialAngle = -102.5 + 192.5
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint5InitialAngle = -165 + 75
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";

      Body.Manipulator youBotManipulatorBody(
        joint5InitialAngle=joint5InitialAngle,
        joint4InitialAngle=joint4InitialAngle,
        joint3InitialAngle=joint3InitialAngle,
        joint2InitialAngle=joint2InitialAngle,
        joint1InitialAngle=joint1InitialAngle,
        manipulatorAnimation=manipulatorAnimation,
        jointFrameOfReference=jointFrameOfReference,
        endPointFrameOfReference=endPointFrameOfReference,
        endPointAnimation=endPointAnimation,
        baseX=baseX,
        baseY=baseY,
        worldAnimation=worldAnimation)
                                  annotation (Placement(transformation(extent={
                {40,-20},{80,20}}, rotation=0)));
      Axis.VRampAxis vRampAxis5(
        nominalVoltage=24,
        nominalCurrent=0.49,
        armatureResistance=13.7,
        armatureInductance=0.00773,
        motorInertia=0.0000035,
        gearboxInertia=0.000000071,
        encoderTickPerRound=4000,
        thresholdVelocity=0,
        maximumCurrent=1.75,
        PParam1Velocity=200,
        PParam2Velocity=800,
        IParam1Velocity=200,
        IParam2Velocity=600,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        maximumPWM=32767,
        PParamCurrent=4000,
        IParamCurrent=4000,
        DParamCurrent=0,
        IClipCurrent=1000,
        thresholdPosition=0,
        PParam1Position=50,
        PParam2Position=50,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        positionTolerance=5,
        velocityTolerance=0.737463,
        acceleration=2.94985,
        maximumVelocity=5.8997,
        nominalRPM=2800,
        gearboxRatio=71,
        startTime=1)         annotation (Placement(transformation(extent={{-40,
                50},{-20,70}}, rotation=0)));
      Axis.VRampAxis vRampAxis4(
        nominalVoltage=24,
        nominalCurrent=1.07,
        nominalRPM=2850,
        armatureResistance=4.48,
        armatureInductance=0.00224,
        motorInertia=0.00000925,
        gearboxInertia=0.000000071,
        gearboxRatio=71,
        encoderTickPerRound=4000,
        maximumVelocity=5.8997,
        thresholdVelocity=0,
        maximumCurrent=2.746,
        PParam1Velocity=200,
        PParam2Velocity=1000,
        IParam1Velocity=200,
        IParam2Velocity=2000,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        maximumPWM=32767,
        PParamCurrent=3000,
        IParamCurrent=3000,
        DParamCurrent=0,
        IClipCurrent=1000,
        thresholdPosition=0,
        PParam1Position=200,
        PParam2Position=200,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        velocityTolerance=0.737463,
        positionTolerance=5,
        acceleration=2.94985,
        startTime=1)         annotation (Placement(transformation(extent={{-40,
                20},{-20,40}}, rotation=0)));
      Axis.VRampAxis vRampAxis3(
        nominalVoltage=24,
        nominalCurrent=2.36,
        nominalRPM=5260,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000071,
        gearboxRatio=100,
        encoderTickPerRound=4000,
        maximumVelocity=4.8879,
        thresholdVelocity=0,
        maximumCurrent=3.899,
        PParam1Velocity=200,
        PParam2Velocity=2000,
        IParam1Velocity=200,
        IParam2Velocity=4000,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        maximumPWM=32767,
        PParamCurrent=1500,
        IParamCurrent=1500,
        DParamCurrent=0,
        IClipCurrent=1000,
        thresholdPosition=0,
        PParam1Position=200,
        PParam2Position=200,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        positionTolerance=5,
        velocityTolerance=0.523599,
        acceleration=2.0944,
        startTime=1)         annotation (Placement(transformation(extent={{-40,
                -10},{-20,10}}, rotation=0)));
      Axis.VRampAxis vRampAxis2(
        nominalVoltage=24,
        nominalCurrent=2.36,
        nominalRPM=5260,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000409,
        gearboxRatio=156,
        encoderTickPerRound=4000,
        maximumVelocity=2.68512,
        thresholdVelocity=0,
        maximumCurrent=5.768,
        PParam1Velocity=200,
        PParam2Velocity=2000,
        IParam1Velocity=200,
        IParam2Velocity=6000,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        maximumPWM=32767,
        PParamCurrent=1500,
        IParamCurrent=1500,
        DParamCurrent=0,
        IClipCurrent=1000,
        thresholdPosition=0,
        PParam1Position=200,
        PParam2Position=200,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        positionTolerance=5,
        velocityTolerance=0.33564,
        acceleration=1.34256,
        startTime=1)         annotation (Placement(transformation(extent={{-40,
                -40},{-20,-20}}, rotation=0)));
      Axis.VRampAxis vRampAxis1(
        nominalVoltage=24,
        nominalCurrent=2.36,
        nominalRPM=5260,
        armatureResistance=0.978,
        armatureInductance=0.000573,
        motorInertia=0.0000135,
        gearboxInertia=0.000000409,
        gearboxRatio=156,
        encoderTickPerRound=4000,
        maximumVelocity=2.68512,
        thresholdVelocity=0,
        maximumCurrent=5.768,
        PParam1Velocity=200,
        PParam2Velocity=2500,
        IParam1Velocity=200,
        IParam2Velocity=2000,
        DParam1Velocity=0,
        DParam2Velocity=0,
        IClip1Velocity=500,
        IClip2Velocity=1000,
        maximumPWM=32767,
        PParamCurrent=1200,
        IParamCurrent=1000,
        DParamCurrent=0,
        IClipCurrent=1000,
        thresholdPosition=0,
        PParam1Position=200,
        PParam2Position=200,
        IParam1Position=0,
        IParam2Position=0,
        DParam1Position=0,
        DParam2Position=0,
        IClip1Position=500,
        IClip2Position=1000,
        positionTolerance=5,
        velocityTolerance=0.33564,
        acceleration=1.34256,
        startTime=1)         annotation (Placement(transformation(extent={{-40,
                -70},{-20,-50}}, rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint5SetPosition
        annotation (Placement(transformation(extent={{-90,50},{-70,70}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint4SetPosition
        annotation (Placement(transformation(extent={{-90,20},{-70,40}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint3SetPosition
        annotation (Placement(transformation(extent={{-90,-10},{-70,10}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint2SetPosition
        annotation (Placement(transformation(extent={{-90,-40},{-70,-20}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint1SetPosition
        annotation (Placement(transformation(extent={{-90,-70},{-70,-50}},
              rotation=0)));
    equation
      connect(vRampAxis5.flange_b, youBotManipulatorBody.joint5)
                                                             annotation (Line(
            points={{-21.4286,60},{20,60},{20,12},{44,12}}, color={0,0,0}));
      connect(vRampAxis4.flange_b, youBotManipulatorBody.joint4)
                                                             annotation (Line(
            points={{-21.4286,30},{10,30},{10,6},{44,6}}, color={0,0,0}));
      connect(vRampAxis3.flange_b, youBotManipulatorBody.joint3)
                                                             annotation (Line(
            points={{-21.4286,0},{44,0}}, color={0,0,0}));
      connect(vRampAxis2.flange_b, youBotManipulatorBody.joint2)
                                                             annotation (Line(
            points={{-21.4286,-30},{10,-30},{10,-6},{44,-6}}, color={0,0,0}));
      connect(vRampAxis1.flange_b, youBotManipulatorBody.joint1)
                                                             annotation (Line(
            points={{-21.4286,-60},{20,-60},{20,-12},{44,-12}}, color={0,0,0}));
      connect(joint5SetPosition, vRampAxis5.setPosition) annotation (Line(
            points={{-80,60},{-38.5714,60}}, color={0,0,127}));
      connect(joint4SetPosition, vRampAxis4.setPosition) annotation (Line(
            points={{-80,30},{-38.5714,30}}, color={0,0,127}));
      connect(joint3SetPosition, vRampAxis3.setPosition) annotation (Line(
            points={{-80,0},{-38.5714,0}}, color={0,0,127}));
      connect(joint2SetPosition, vRampAxis2.setPosition) annotation (Line(
            points={{-80,-30},{-38.5714,-30}}, color={0,0,127}));
      connect(joint1SetPosition, vRampAxis1.setPosition) annotation (Line(
            points={{-80,-60},{-38.5714,-60}}, color={0,0,127}));
      annotation (Diagram(graphics),
                           Icon(graphics={
            Rectangle(
              extent={{-100,100},{100,-98}},
              lineColor={0,0,255},
              fillColor={255,255,255},
              fillPattern=FillPattern.Solid),
            Bitmap(extent={{-80,80},{80,-60}}, fileName=
                                                "icon_complete.png"),
            Text(
              extent={{-60,-60},{80,-100}},
              lineColor={0,0,255},
              textString=
                   "VRamp Mode")}));
    end ManipulatorVRampMode;

    model ManipulatorDummyMode
      import Modelica.SIunits;
      parameter SIunits.Length baseX = 0
        "| Initial Pose | Location | location of base_manipulator in x axis";
      parameter SIunits.Length baseY = 0
        "| Initial Pose | Location | location of base_manipulator in y axis";
      parameter Boolean endPointFrameOfReference = false
        "| Animation | Animation | Frame of reference for manipulator end point";
      parameter Boolean endPointAnimation = true
        "| Animation | Animation | show manipulator end point animation";
      parameter Boolean manipulatorAnimation = true
        "| Animation | Animation | show manipulator animation";
      parameter Boolean jointFrameOfReference = false
        "| Animation | Animation | show frame of reference for each joint";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint5InitialAngle = -165 + 75
        "| Initial Pose | Initial Angles | Initial angle for joint 5 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint4InitialAngle = -102.5 + 192.5
        "| Initial Pose | Initial Angles | Initial angle for joint 4 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint3InitialAngle = 146 - 236
        "| Initial Pose | Initial Angles | Initial angle for joint 3 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint2InitialAngle = -65 + 155
        "| Initial Pose | Initial Angles | Initial angle for joint 2 at start of simulation";
      parameter Modelica.SIunits.Conversions.NonSIunits.Angle_deg
        joint1InitialAngle = -169 + 79
        "| Initial Pose | Initial Angles | Initial angle for joint 1 at start of simulation";
      parameter Boolean worldAnimation = true
        "| Animation | Animation | show world frame of reference animation";

      Modelica.Blocks.Interfaces.RealInput joint5InputSpeed
        annotation (Placement(transformation(extent={{-90,50},{-70,70}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint4InputSpeed
        annotation (Placement(transformation(extent={{-90,20},{-70,40}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint3InputSpeed
        annotation (Placement(transformation(extent={{-90,-10},{-70,10}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint2InputSpeed
        annotation (Placement(transformation(extent={{-90,-40},{-70,-20}},
              rotation=0)));
      Modelica.Blocks.Interfaces.RealInput joint1InputSpeed
        annotation (Placement(transformation(extent={{-90,-70},{-70,-50}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sources.Speed
                                          speed1(useSupport=false)
        annotation (Placement(transformation(extent={{-40,-70},{-20,-50}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sources.Speed
                                          speed2(useSupport=false)
        annotation (Placement(transformation(extent={{-40,-40},{-20,-20}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sources.Speed
                                          speed3(useSupport=false)
        annotation (Placement(transformation(extent={{-40,-10},{-20,10}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sources.Speed
                                          speed4(useSupport=false)
        annotation (Placement(transformation(extent={{-40,20},{-20,40}},
              rotation=0)));
      Modelica.Mechanics.Rotational.Sources.Speed
                                          speed5(useSupport=false)
        annotation (Placement(transformation(extent={{-40,50},{-20,70}},
              rotation=0)));
      Body.Manipulator youBotManipulatorBody(
        joint5InitialAngle=joint5InitialAngle,
        joint4InitialAngle=joint4InitialAngle,
        joint3InitialAngle=joint3InitialAngle,
        joint2InitialAngle=joint2InitialAngle,
        joint1InitialAngle=joint1InitialAngle,
        manipulatorAnimation=manipulatorAnimation,
        jointFrameOfReference=jointFrameOfReference,
        endPointFrameOfReference=endPointFrameOfReference,
        endPointAnimation=endPointAnimation,
        baseX=baseX,
        baseY=baseY,
        link1(body(color={255,65,65})),
        link2(body(color={255,65,65})),
        link3(body(color={255,65,65})),
        link4(body(color={255,65,65})),
        link5(body(color={255,65,65})),
        worldAnimation=worldAnimation)
        annotation (Placement(transformation(extent={{40,-20},{80,20}},
              rotation=0)));
    equation
      connect(joint5InputSpeed, speed5.w_ref)   annotation (Line(points={{-80,
              60},{-42,60}}, color={0,0,127}));
      connect(joint4InputSpeed, speed4.w_ref)   annotation (Line(points={{-80,
              30},{-42,30}}, color={0,0,127}));
      connect(joint3InputSpeed, speed3.w_ref)   annotation (Line(points={{-80,0},
              {-42,0}}, color={0,0,127}));
      connect(joint2InputSpeed, speed2.w_ref)   annotation (Line(points={{-80,
              -30},{-42,-30}}, color={0,0,127}));
      connect(joint1InputSpeed, speed1.w_ref)   annotation (Line(points={{-80,
              -60},{-42,-60}}, color={0,0,127}));
      connect(speed5.flange,   youBotManipulatorBody.joint5)
                                                   annotation (Line(points={{
              -20,60},{20,60},{20,12},{44,12}}, color={0,0,0}));
      connect(speed4.flange,   youBotManipulatorBody.joint4)
                                                   annotation (Line(points={{
              -20,30},{10,30},{10,6},{44,6}}, color={0,0,0}));
      connect(speed3.flange,   youBotManipulatorBody.joint3)
        annotation (Line(points={{-20,0},{44,0}}, color={0,0,0}));
      connect(speed2.flange,   youBotManipulatorBody.joint2)
                                                   annotation (Line(points={{
              -20,-30},{10,-30},{10,-6},{44,-6}}, color={0,0,0}));
      connect(speed1.flange,   youBotManipulatorBody.joint1)
                                                   annotation (Line(points={{
              -20,-60},{20,-60},{20,-12},{44,-12}}, color={0,0,0}));
      annotation (Diagram(graphics),
                           Icon(graphics={
            Rectangle(
              extent={{-100,100},{102,-100}},
              lineColor={0,0,255},
              fillColor={255,255,255},
              fillPattern=FillPattern.Solid),
            Text(
              extent={{-60,-60},{80,-100}},
              lineColor={0,0,255},
              textString=
                   "Dummy Mode"),
            Bitmap(extent={{-78,80},{82,-60}}, fileName=
                                               "icon_dummy.png")}));
    end ManipulatorDummyMode;
  end System;

  annotation (uses(Modelica(version="3.2")),
    version="1",
    conversion(from(version="", script="ConvertFromyouBot_.mos")));
end youBot;
